% Compute the element load vectors.
%
% function evs = gap_loads(self, geom, u)
%
% Return an array of the load vectors so they may be assembled.
%    Call as
% evs = distrib_loads(feb, geom, u, fi, m)
%     geom=geometry field
%     u=displacement field
%     fi=force intensity object
%     m= manifold dimension
%
function evs = gap_loads(self, geom, u)
    gcells = get(self.feblock,'gcells');
    ngcells = length(gcells);
    nfens = get(gcells(1),'nfens');
    dim = get(u,'dim');
    % Integration rule
    integration_rule = get(self.feblock, 'integration_rule');
    pc = get(integration_rule, 'param_coords');
    w  = get(integration_rule, 'weights');
    npts_per_gcell = get(integration_rule, 'npts');
    conns = get(gcells, 'conn'); % connectivity
    %     Preallocate
    Fe = cell(size(conns,1),1);
    eqnums = cell(size(conns,1),1);
    for i=1:size(conns,1)
        eqnums{i} =gather(u,conns(i,:),'eqnums');
        Fe{i} =zeros(get(geom,'dim')*nfens,1);
    end
    xs =get(geom,'values');
    % Now loop over all gcells in the block
    for i=1:ngcells
        conn =conns(i,:);
        X=xs(conn,:);
        U = gather(u, conn, 'values', 'noreshape'); % displacement in step n
        x = X + U; % previous known coordinates
        gap = self.get_gap (self.surface_data,x);
        if (gap<0)
            normal = self.get_normal (self.surface_data,x);
            normal = normal/norm(normal);
             Fe{i} = Fe{i} - normal'*gap*self.penalty;
        end
    end
    evs  = elevecset(struct('vec',{Fe}, 'eqnums',{eqnums}));
end
