% Compute the stiffness matrices of the individual gcells.
%
% function ems = stiffness (self, geom, u)
%
% Return an array of the element matrices so they may be assembled.
%     geom=geometry field
%     u=displacement field
function ems = infsup_Gh (self, geom, u)
    gcells = get(self.feblock,'gcells');
    nfens = get(gcells,'nfens');
    % Integration rule
    integration_rule = get(self.feblock, 'integration_rule');
    pc = get(integration_rule, 'param_coords');
    w  = get(integration_rule, 'weights');
    npts_per_gcell = get(integration_rule, 'npts');
    for j=1:npts_per_gcell
        Ns{j} = bfun(gcells,pc(j,:));
        Nders{j} = bfundpar(gcells,pc(j,:));
    end
    % Material orientation /directions
    Rm = get(self,'Rm');
    eval_Rm = 0;
    if strcmp(class(Rm),'function_handle')
        eval_Rm = true; Rmh =Rm;
    end
    % Material
    mat = get(self.feblock, 'mater');
    % Now loop over all gcells in the block
    conns = get(gcells, 'conn'); % connectivity
    Ke = cell(size(conns,1),1);
    eqnums = cell(size(conns,1),1);
    for i=1:size(conns,1)
        eqnums{i} =gather(u,conns(i,:),'eqnums');
        Ke{i} =zeros(get(geom,'dim')*nfens);
    end
    xs =gather(geom,(1:get(geom,'nfens')),'values','noreshape');
    for i=1:length(gcells)
        conn =conns(i,:);
        x=xs(conn,:);
        % Loop over all integration points
        for j=1:npts_per_gcell
            c =Ns{j}'*x;% physical location of the quadrature point
            J = x' * Nders{j};% We compute the Jacobian matrix
            if (eval_Rm)% do I need to evaluate the local material orientation?
                Rm =Rmh(c,J);
            end
            if (isempty(Rm))
                Ndersp = Nders{j}/J;% derivatives wrt local coor
            else
                Ndersp = Nders{j}/(Rm'*J);% derivatives wrt local coor
            end
            Jac = Jacobian_volume(gcells,conn, Ns{j}, J, x);
            if (Jac<=0),error('Non-positive Jacobian');end
            B = self.hdivmat(self,Ns{j},Ndersp,c,Rm);% strain-displacement
            Ke{i} = Ke{i} + B'*B * (Jac * w(j));
        end
    end
    ems  = elematset(struct('mat',{Ke}, 'eqnums',{eqnums}));
end