%  Draw graphic representation of local coordinate system for all beam gcells.
%
% function draw_local_coordinates(self, gv, context)
%
%
% Input arguments
% self = self
% gv = graphic viewer
% context = struct
% with typically mandatory fields
%    x=reference geometry field
%    u=displacement field
% and with optional fields
%    length = length of the arrows
%
function ems = draw_local_coordinates (self, gv, context)
    if (isfield(context,'length'))
        a=context.length;
    else
        a=1;
    end
    gcells = get(self.feblock,'gcells');
    ngcells = count(gcells);
    nfens = get(gcells(1),'nfens');
    dim = get(context.x,'dim');
    % Now loop over all gcells in the block
    conns = get(gcells, 'conn'); % connectivity
    Xs =gather(context.x,(1:get(context.x,'nfens')),'values','noreshape');
    x1x2_vector=get(gcells,'x1x2_vector');
    % Now loop over all gcells in the block
    for i=1:ngcells
        conn = conns(i,:); % connectivity
        x = Xs(conn,:); % coordinates of nodes
        L=center_line_length(gcells,x);
        T= local_coordinate_system (gcells, x, x1x2_vector(:,i));
        Center=mean(x);
        options.color='red';
        draw_arrow(gv, Center, a*T(:,1), options);
        options.color='green';
        draw_arrow(gv, Center, a*T(:,2), options);
        options.color='blue';
        draw_arrow(gv, Center, a*T(:,3), options);
    end
end