% Compute the mass matrices of the individual beam gcells.
% Return an array of them so they may be assembled.
%    Call as
% ems = mass(feb, geom, u)
%     geom=geometry field
%     ur=displacement+rotation field
%
function ems = mass (self, geom, u)
    gcells = get(self.feblock,'gcells');
    ngcells = count(gcells);
    nfens = get(gcells(1),'nfens');
    dim = get(geom,'dim');
    % Pre-allocate the element matrices
    % Now loop over all gcells in the block
    conns = get(gcells, 'conn'); % connectivity
    Me = cell(size(conns,1),1);
    eqnums = cell(size(conns,1),1);
    for i=1:size(conns,1)
        eqnums{i} =gather(u,conns(i,:),'eqnums');
        Me{i} =zeros(6*nfens);
    end
    Xs =gather(geom,(1:get(geom,'nfens')),'values','noreshape');
    % Material
    mat = get(self.feblock, 'mater');
    prop=get(mat, 'property' );
    E=get(prop , 'E' );
    nu=get(prop , 'nu' );
    G = E/2/(1+nu);
    rho=get(prop , 'rho' );
    %     matstates = get(self.feblock, 'matstates');
    % Section properties
    A=get(gcells,'A');
    I2=get(gcells,'I2');
    I3=get(gcells,'I3');
    IT=get(gcells,'IT');
    x1x2_vector=get(gcells,'x1x2_vector');
    % Now loop over all gcells in the block
    Z=zeros(3,3);
    for i=1:ngcells
        conn = conns(i,:); % connectivity
        x = Xs(conn,:); % coordinates of nodes
        L=center_line_length(gcells,x);
        Me{i}=local_mass(self, A(i), I2(i), I3(i), IT(i), rho, L);
        T= local_coordinate_system (gcells, x, x1x2_vector(:,i));
        Te=[T Z Z Z; ...
            Z T Z Z;...
            Z Z T Z;...
            Z Z Z T];
        Me{i} = Te*Me{i}*Te';
    end
     ems  = elematset(struct('mat',{Me}, 'eqnums',{eqnums}));
end
