% Compute the stiffness matrices of the individual beam gcells.
% Return an array of them so they may be assembled.
%    Call as
% ems = stiffness(feb, geom, u)
%     geom=geometry field
%     ur=displacement+rotation field
%
function ems = stiffness_geo (self, geom, u)
    gcells = get(self.feblock,'gcells');
    ngcells = count(gcells);
    nfens = get(gcells(1),'nfens');
    dim = get(geom,'dim');
    % Pre-allocate the element matrices
    % Now loop over all gcells in the block
    conns = get(gcells, 'conn'); % connectivity
    Kg = cell(size(conns,1),1);
    eqnums = cell(size(conns,1),1);
    for i=1:size(conns,1)
        eqnums{i} =gather(u,conns(i,:),'eqnums');
        Kg{i} =zeros(6*nfens);
    end
    Xs =gather(geom,(1:get(geom,'nfens')),'values','noreshape');
    Us =gather(u,(1:get(geom,'nfens')),'values','noreshape');
    % Material
    mat = get(self.feblock, 'mater');
    prop=get(mat, 'property' );
    E=get(prop , 'E' );% Young's modulus
    G=get(prop , 'G' );% shear modulus
    %     matstates = get(self.feblock, 'matstates');
    % Section properties
    A=get(gcells,'A');
    I2=get(gcells,'I2');
    I3=get(gcells,'I3');
    IT=get(gcells,'IT');
    x1x2_vector=get(gcells,'x1x2_vector');
    % Now loop over all gcells in the block
    Z=zeros(3,3);
    for i=1:ngcells
        conn = conns(i,:); % connectivity
        x = Xs(conn,:); % coordinates of nodes
        UI = [Us(conn(1),1:3)';Us(conn(1),4:6)'];
        UJ = [Us(conn(2),1:3)';Us(conn(2),4:6)'];
        L=center_line_length(gcells,x);
        T= local_coordinate_system (gcells, x, x1x2_vector(:,i));
        Te=[T Z Z Z; ...
            Z T Z Z;...
            Z Z T Z;...
            Z Z Z T];
        LU= Te'*[UI;UJ];
        PN = natural_forces (self, E, G, A(i), I2(i), I3(i), IT(i), L, LU);
        Kg{i}=local_geometric_stiffness(self, A(i), I2(i), I3(i), PN, L);
        Kg{i} = Te*Kg{i}*Te';
    end
     ems  = elematset(struct('mat',{Kg}, 'eqnums',{eqnums}));
end