% Compute the stiffness matrices of the individual beam gcells.
% Return an array of them so they may be assembled.
%    Call as
% ems = stiffness(feb, geom, u)
%     geom=geometry field
%     ur=displacement+rotation field
%
function ems = coriolis_x_only (self, geom, u1, Rfield, Omega_x)
    gcells = get(self.feblock,'gcells');
    ngcells = count(gcells);
    nfens = get(gcells(1),'nfens');
    dim = get(geom,'dim');
    % Pre-allocate the element matrices
    % Now loop over all gcells in the block
    conns = get(gcells, 'conn'); % connectivity
    Kg = cell(size(conns,1),1);
    eqnums = cell(size(conns,1),1);
    for i=1:size(conns,1)
        eqnums{i} =gather(u1,conns(i,:),'eqnums');
        Kg{i} =zeros(6*nfens);
    end
    Xs =gather(geom,(1:get(geom,'nfens')),'values','noreshape');
    Rs = gather(Rfield, (1:get(geom,'nfens')), 'values', 'noreshape'); % displacement in step n
    U1s =gather(u1,(1:get(geom,'nfens')),'values','noreshape');
    % Material
    mat = get(self.feblock, 'mater');
    prop=get(mat, 'property' );
    E=get(prop , 'E' );% Young's modulus
    G=get(prop , 'G' );% shear modulus
    rho=get(prop , 'rho' );
    %     matstates = get(self.feblock, 'matstates');
    % Section properties
    A=get(gcells,'A');
    I2=get(gcells,'I2');
    I3=get(gcells,'I3');
    IT=get(gcells,'IT');
    x1x2_vector=get(gcells,'x1x2_vector');
    % Now loop over all gcells in the block
    Z=zeros(3,3);
    for i=1:ngcells
        conn = conns(i,:); % connectivity
        x0 = Xs(conn,:); % coordinates of nodes
        U1 = U1s(conn,:); % coordinates of nodes
        RI =reshape(Rs(conn(1),:),3,3);
        RJ =reshape(Rs(conn(2),:),3,3);
        x1 = x0+U1s(conn,1:3); % coordinates of nodes
        [L1,T,rhoN]=local_frames(x0,x1x2_vector(:,i),x1,RI,RJ);
        Te=[T Z Z Z; ...
            Z T Z Z;...
            Z Z T Z;...
            Z Z Z T];
        Kg{i}=local_coriolis(self,rho,A(i),IT(i),Omega_x,L1);
        Kg{i} = Te*Kg{i}*Te';
    end
    ems  = elematset(struct('mat',{Kg}, 'eqnums',{eqnums}));
end