% Compute element force vectors for mass proportional terms (inertial
% force).
%
% function evs = inertial_force(self, geom, u, ems)
%
% Compute the element load vectors corresponding to nonzero essential boundary
% conditions (settlement of the supports).
% Return an array of them so they may be assembled.
%    Call as
% function evs = inertial_force(self, geom, u, ems)
% or
% function evs = inertial_force(self, geom, u)
%     geom=geometry field
%     u=displacement field
%
function evs = inertial_force(self, geom, u, Rfield, ems)
    if (~exist('ems','var'))
        ems = mass (self, geom, u, Rfield);
    end
    gcells = get(self.feblock,'gcells');
    nfens = get(gcells,'nfens');
    dim = get(u,'dim');
    % Pre-allocate the element matrices
    conns = get(gcells, 'conn'); % connectivity
    Fe = cell(size(conns,1),1);
    eqnums = cell(size(conns,1),1);
    for i=1:size(conns,1)
        eqnums{i} =gather(u,conns(i,:),'eqnums');
        Fe{i} =zeros(dim*nfens,1);
    end
    mat=get(ems,'mat'); 
    % Now loop over all gcells in the block
    for i=1:count(gcells)
        conn = conns(i,:); % connectivity
        pu = gather(u, conn, 'values');
        if norm (pu) ~= 0
            Fe{i} = mat{i}*pu;
        end
    end
    evs  = elevecset(struct('vec',{Fe}, 'eqnums',{eqnums}));
end