% C  PROCEDURE CALCULATES ELASTIC COEFFICIENTS OF STIFFNESS MATRIX
% C  OF TWO-NODE BEAM ELEMENT
% C     L.........LENGTH OF BEAM AXIS
% C     IN2, IN3..MOMENTS OF INERTIA, BENDING ABOUT THE X2-AXIS (IN2)
% C               OR X3-AXIS (IN3)
% C     IT... moment of inertia for torsion (about the local X1 axis)
% Output
% C     SM ... local stiffness matrix, 12 x 12
function SM = local_stiffness(self, EY, G, CSA, IN2, IN3, IT, L)
    aN = local_cartesian_to_natural (self, L);
    DN= diag([EY*CSA/L,
        EY*IN3/L,
        3*EY*IN3/L,
        EY*IN2/L,
        3*EY*IN2/L,
        G*IT/L]);
    SM=aN'*DN*aN;
end

%     Grbm = rigid_body_matrix(self,L);
%     Db= [EY*CSA/L,0,0,0,0,0;
%         0,G*IT/L,0,0,0,0;
%         0,0,4*EY*IN2/L,2*EY*IN2/L,0,0;
%         0,0,2*EY*IN2/L,4*EY*IN2/L,0,0;
%         0,0,0,0,4*EY*IN3/L,2*EY*IN3/L;
%         0,0,0,0,2*EY*IN3/L,4*EY*IN3/L];
%     SM=Grbm'*Db*Grbm;
   
%     Symbolic derivation possible with the code
%    syms EY CSA L IN2 IN3 TST real
% Grbm = [-1,0,0,0,0,0,1,0,0,0,0,0;
%         0,0,0,-1,0,0,0,0,0,1,0,0;
%         0,0,-1/L,0,1,0,0,0,1/L,0,0,0;
%         0,0,-1/L,0,0,0,0,0,1/L,0,1,0;
%         0,1/L,0,0,0,1,0,-1/L,0,0,0,0;
%         0,1/L,0,0,0,0,0,-1/L,0,0,0,1;
%        ];
%    Db= [EY*CSA/L,0,0,0,0,0;
%        0,TST/L,0,0,0,0;
%        0,0,4*EY*IN2/L,2*EY*IN2/L,0,0;
%        0,0,2*EY*IN2/L,4*EY*IN2/L,0,0;
%        0,0,0,0,4*EY*IN3/L,2*EY*IN3/L;
%        0,0,0,0,2*EY*IN3/L,4*EY*IN3/L];
%    SM=Grbm'*Db*Grbm
%    
%    simple (SM)
