% Compute the damping matrices of the individual gcells.
% Return an array of them so they may be assembled.
%    Call as
% ems = damping(feb, geom, v)
%     geom=geometry field
%     v=velocity field
%
function ems = damping (self, geom, v)
    gcells = get(self,'gcells');
    ngcells = length(gcells);
    nfens = get(gcells(1),'nfens');
    dim = get(geom,'dim');
    % Pre-allocate the element matrices
    ems(1:ngcells) = deal(elemat);
    % Integration rule
    integration_rule = get(self, 'integration_rule');
    % - shear terms
    pc_s = get(integration_rule, 'param_coords');
    w_s  = get(integration_rule, 'weights');
    n_s = get(integration_rule, 'npts');
    % Material
    mat = get(self, 'mater');
    % Loop over all the geometry cells
    for i=1:ngcells
        conn = get(gcells(i), 'conn'); % connectivity
        x = gather(geom, conn, 'values', 'noreshape'); % coord
        Ke = zeros(dim*nfens); % element stiffness matrix
        for j=1:n_s
            Rm=material_directions(self,gcells(i),pc_s(j,:),x);
            detJ= Jacobian_volume(gcells(i),pc_s(j,:),x);
            B = Blmat(gcells(i), pc_s(j,:), x, Rm);
            Dv = damping_moduli (mat, struct([]));
            Ke = Ke + B'*Dv*B * detJ * w_s(j);
        end
        ems(i) = set(ems(i), 'mat', Ke);
        ems(i) = set(ems(i), 'eqnums',gather(v,conn,'eqnums'));
    end
    return;
end