% Compute the lumped volume matrices of the individual gcells. Requires the
% use of a nodal quadrature rule!
%
% function ems = lumped_volume(self, geom, temp)
%
% Return an array of them so they may be assembled.
%  
%    where
%       geom=geometry field
%       temp = field used to retrieve equation numbers   
% 
function ems = lumped_volume(self, geom, temp)
    gcells = get(self.feblock_diffusion,'gcells');
    ngcells = length(gcells);
    nfens = get(gcells(1),'nfens');
    % Integration rule
    integration_rule = get(self.feblock_diffusion, 'integration_rule');
    pc = get(integration_rule, 'param_coords');
    w  = get(integration_rule, 'weights');
    npts_per_gcell = get(integration_rule, 'npts');
    for j=1:npts_per_gcell
        N{j} = bfun(gcells,pc(j,:));
        Nder{j} = bfundpar(gcells,pc(j,:));
    end
    % Material
    mat = get(self.feblock_diffusion, 'mater');
    conns = get(gcells, 'conn'); % connectivity
    Ce = cell(size(conns,1),1);
    eqnums = cell(size(conns,1),1);
    for i=1:size(conns,1)
        eqnums{i} =gather(temp,conns(i,:),'eqnums');
        Ce{i} =zeros(nfens);
    end
    xs =gather(geom,(1:get(geom,'nfens')),'values','noreshape');
    % Now loop over all gcells in the block
    for i=1:ngcells
        conn = conns(i,:); % connectivity
        x = xs(conn, :); % coord
        for j=1:npts_per_gcell % Loop over integration points
            J = Jacobian_matrix(gcells,Nder{j},x);
            Jac = Jacobian_volume(gcells,conn, N{j}, J, x);
            Ce{i} = Ce{i} + N{j}*N{j}' * (Jac * w(j));
        end
    end
    ems  = elematset(struct('mat',{Ce}, 'eqnums',{eqnums}));
end
