% Compute the surface heat transfer matrices of the individual gcells.
%
% function ems = surface_transfer(self, geom, temp)
%
% Return an array of the element matrices so they may be assembled.
%    Call as     ems = surface_transfer(feb, geom, temp)
%    where    geom=geometry field
%             temp=temperature field
% 
function ems = surface_transfer(self, geom, temp)
    gcells = get(self.feblock_diffusion,'gcells');
    ngcells = length(gcells);
    nfens = get(gcells(1),'nfens');
    % Integration rule
    integration_rule = get(self.feblock_diffusion, 'integration_rule');
    pc = get(integration_rule, 'param_coords');
    w  = get(integration_rule, 'weights');
    npts_per_gcell = get(integration_rule, 'npts');
    % surface transfer coefficient
    h = get(self.feblock_diffusion,'surface_transfer');
    if (strcmp(class(h),'function_handle'))
        need_to_eval=1;
    else
        need_to_eval=0; H = h;
    end
    conns = get(gcells, 'conn'); % connectivity
    Ke = cell(size(conns,1),1);
    eqnums = cell(size(conns,1),1);
    for i=1:size(conns,1)
        eqnums{i} =gather(temp,conns(i,:),'eqnums');
        Ke{i} =zeros(nfens);
    end
    xs =get(geom,'values');
    Ts =gather(temp,(1:get(geom,'nfens')),'values','noreshape');
    for j=1:npts_per_gcell
        N = bfun(gcells,pc(j,:));
        Nder = bfundpar(gcells,pc(j,:));
        for i=1:size(conns,1)
            conn =conns(i,:);
            x=xs(conn,:);
            T = Ts(conn, :);% node temperatures
            J = Jacobian_matrix(gcells,Nder,x);
            Jac = Jacobian_surface(gcells,conn, N, J, x);
            if (need_to_eval)
                H = h(N'*T);
            end
            Ke{i} = Ke{i} + ((H*Jac*w(j))*N)*N';
        end
    end
    ems  = elematset(struct('mat',{Ke}, 'eqnums',{eqnums}));
end
