% Draw a graphic representation two-manifold geometric cell.
%
% function draw (self, gv, context)
%
%
% Input arguments
% self = self
% gv = graphic viewer
% context = struct
% with mandatory fields
%    x=reference geometry field
%    u=displacement field
% with optional fields
%    facecolor = color of the faces, solid
%    colorfield =field with vertex colors
%       only one of the facecolor and colorfield may be supplied
%    shrink = shrink factor
%
function draw (self, gv, context)
    ef=context.edges;
    f=context.faces;
    conns = get(self, 'conn'); % connectivity
    if (isempty(conns))
        return; % nothing to be done
    end
    shrink  = 1.0;
    if isfield(context,'shrink')
        shrink =context.shrink;
    end
    if (shrink==1)
        nn =max(max(conns));
        x = gather(context.x, 1:nn, 'values', 'noreshape'); % coordinates of nodes
        u = gather(context.u, 1:nn, 'values', 'noreshape'); % coordinates of nodes
        xu=x+u;
        if (isfield(context,'colorfield'))
            colors =gather(context.colorfield, 1:nn, 'values', 'noreshape'); % coordinates of nodes
            options.colors=colors;
            options.facecolor = 'none';
        else
            options.facecolor = 'red';
            if (isfield(context,'facecolor'))
                options.facecolor = context.facecolor;
            end
        end
        if (isfield(context,'facealpha'))
            options.facealpha = context.facealpha;
        end
        options.edgecolor ='none';
        for i=1:size(f,1)
            draw_polygon (gv, xu, conns(:,f(i,:)), options);
        end
        options.edgecolor='black';
        if (isfield(context,'edgecolor'))
            options.edgecolor =context.edgecolor;
        end
        if ~strcmp(options.edgecolor,'none')
            draw_polyline (gv, xu, conns(:,ef), options);
        end
    else
        nfens=size(conns,2);
        for ac=1:size(conns,1)
            conn =conns(ac,:);
            x = gather(context.x, conn, 'values', 'noreshape'); % coordinates of nodes
            u = gather(context.u, conn, 'values', 'noreshape'); % coordinates of nodes
            xu=x+u;
            xus=xu;
            for j=1:length(context.shrunk_pc_N)
                xus(j,:)=context.shrunk_pc_N{j}'*xu;
            end
            xu=xus;
            if (isfield(context,'colorfield'))
                colors =gather(context.colorfield, conn, 'values', 'noreshape'); % coordinates of nodes
                options.colors=colors;
                options.facecolor = 'none';
            else
                options.facecolor = 'red';
                if (isfield(context,'facecolor'))
                    options.facecolor = context.facecolor;
                end
            end
            if (isfield(context,'facealpha'))
                options.facealpha = context.facealpha;
            end
            options.edgecolor ='none';
            for i=1:size(f,1)
                draw_polygon (gv, xu, f(i,:), options);
            end
            options.edgecolor='black';
            if (isfield(context,'edgecolor'))
                options.edgecolor =context.edgecolor;
            end
            if ~strcmp(options.edgecolor,'none')
                draw_polyline (gv, xu, ef, options);
            end
        end
    end
end
