% Evaluate the derivatives of the basis function matrix.
%
% function Nder = bfundpar (self, param_coords)
%
% Returns an array of NFENS rows, and DIM columns, where
%    NFENS=number of nodes, and
%    DIM=number of spatial dimensions.
% Call as:
%    Nder = bfundpar(g, pc)
% where g=gcell
%       pc=parametric coordinates, -1 < pc(j) < 1, length(pc)=3.
%
function val = bfundpar (self, param_coords)
    %     idx=[1     2     5     4    10    11    14    13     3     8     6     7    12    17    15    16    19    20    23   22     9    21    26    24    25    18    27];
    %     xi=param_coords(1);
    %     xis = [(xi-1)*xi/2;  (xi+1)*xi/2;  -(xi+1)*(xi-1)];
    %     dxis = [x1; (xi+1/2); -2*xi];
    %     eta=param_coords(2);
    %     etas = [(eta-1)*eta/2;  (eta+1)*eta/2;  -(eta+1)*(eta-1)];
    %     detas=[(eta-1/2); (eta+1/2); -2*eta];
    %     zeta=param_coords(3);
    %     zetas = [(zeta-1)*zeta/2;  (zeta+1)*zeta/2;  -(zeta+1)*(zeta-1)];
    %     dzetas=[(zeta-1/2); (zeta+1/2); -2*zeta];
    %
    %     Nxi =zeros(3, 3, 3);
    %     xisetas=(dxis*etas');
    %     Nxi(:,:,1)=xisetas*zetas(1);
    %     Nxi(:,:,2)=xisetas*zetas(2);
    %     Nxi(:,:,3)=xisetas*zetas(3);
    %
    %     Net =zeros(3, 3, 3);
    %     xisetas=(xis*detas');
    %     Net(:,:,1)=xisetas*zetas(1);
    %     Net(:,:,2)=xisetas*zetas(2);
    %     Net(:,:,3)=xisetas*zetas(3);
    %
    %     Nze =zeros(3, 3, 3);
    %     xisetas=(xis*etas');
    %     Nze(:,:,1)=xisetas*dzetas(1);
    %     Nze(:,:,2)=xisetas*dzetas(2);
    %     Nze(:,:,3)=xisetas*dzetas(3);
    %
    % %     val = [Nxi(idx)',Net(idx)',Nze(idx)'];
    %     val = [Nxi(idx);Net(idx);Nze(idx)]';
    %     return;
    % end
    xi=param_coords(1);
    eta=param_coords(2);
    zet=param_coords(3);
    x1 =(xi-1/2);
    x2 =(xi+1/2);
    x3 =(xi-1);
    x4 =(xi+1);
    z1 =(zet-1);
    z2 =(zet-1/2);
    z3 =(zet+1);
    z4 =(zet+1/2);
    y1 =(eta-1);
    y2 =(eta-1/2);
    y3 =(eta+1);
    y4 = (eta+1/2);
    val = [
        [       1/4*z1*zet*x1*y1*eta,        1/4*z1*zet*x3*xi*y2,        1/4*z2*x3*xi*y1*eta]
        [       1/4*z1*zet*x2*y1*eta,        1/4*z1*zet*x4*xi*y2,        1/4*z2*x4*xi*y1*eta]
        [       1/4*z1*zet*x2*y3*eta,        1/4*z1*zet*x4*xi*y4,        1/4*z2*x4*xi*y3*eta]
        [       1/4*z1*zet*x1*y3*eta,        1/4*z1*zet*x3*xi*y4,        1/4*z2*x3*xi*y3*eta]
        [       1/4*z3*zet*x1*y1*eta,        1/4*z3*zet*x3*xi*y2,        1/4*z4*x3*xi*y1*eta]
        [       1/4*z3*zet*x2*y1*eta,        1/4*z3*zet*x4*xi*y2,        1/4*z4*x4*xi*y1*eta]
        [       1/4*z3*zet*x2*y3*eta,        1/4*z3*zet*x4*xi*y4,        1/4*z4*x4*xi*y3*eta]
        [       1/4*z3*zet*x1*y3*eta,        1/4*z3*zet*x3*xi*y4,        1/4*z4*x3*xi*y3*eta]
        [            -1/2*z1*zet*xi*y1*eta,   1/2*z1*zet*(-x4)*x3*y2,   1/2*z2*(-x4)*x3*y1*eta]
        [  1/2*z1*zet*x2*(-y3)*y1,             -1/2*z1*zet*x4*xi*eta,   1/2*z2*x4*xi*(-y3)*y1]
        [            -1/2*z1*zet*xi*y3*eta,   1/2*z1*zet*(-x4)*x3*y4,   1/2*z2*(-x4)*x3*y3*eta]
        [  1/2*z1*zet*x1*(-y3)*y1,             -1/2*z1*zet*x3*xi*eta,   1/2*z2*x3*xi*(-y3)*y1]
        [            -1/2*z3*zet*xi*y1*eta,   1/2*z3*zet*(-x4)*x3*y2,   1/2*z4*(-x4)*x3*y1*eta]
        [  1/2*z3*zet*x2*(-y3)*y1,             -1/2*z3*zet*x4*xi*eta,   1/2*z4*x4*xi*(-y3)*y1]
        [            -1/2*z3*zet*xi*y3*eta,   1/2*z3*zet*(-x4)*x3*y4,   1/2*z4*(-x4)*x3*y3*eta]
        [  1/2*z3*zet*x1*(-y3)*y1,             -1/2*z3*zet*x3*xi*eta,   1/2*z4*x3*xi*(-y3)*y1]
        [  1/2*(-z3)*z1*x1*y1*eta,   1/2*(-z3)*z1*x3*xi*y2,             -1/2*zet*x3*xi*y1*eta]
        [  1/2*(-z3)*z1*x2*y1*eta,   1/2*(-z3)*z1*x4*xi*y2,             -1/2*zet*x4*xi*y1*eta]
        [  1/2*(-z3)*z1*x2*y3*eta,   1/2*(-z3)*z1*x4*xi*y4,             -1/2*zet*x4*xi*y3*eta]
        [  1/2*(-z3)*z1*x1*y3*eta,   1/2*(-z3)*z1*x3*xi*y4,             -1/2*zet*x3*xi*y3*eta]
        [           -z1*zet*xi*(-y3)*y1,            -z1*zet*(-x4)*x3*eta,  z2*(-x4)*x3*(-y3)*y1]
        [           -(-z3)*z1*xi*y1*eta,  (-z3)*z1*(-x4)*x3*y2,            -zet*(-x4)*x3*y1*eta]
        [ (-z3)*z1*x2*(-y3)*y1,            -(-z3)*z1*x4*xi*eta,            -zet*x4*xi*(-y3)*y1]
        [           -(-z3)*z1*xi*y3*eta,  (-z3)*z1*(-x4)*x3*y4,            -zet*(-x4)*x3*y3*eta]
        [ (-z3)*z1*x1*(-y3)*y1,            -(-z3)*z1*x3*xi*eta,            -zet*x3*xi*(-y3)*y1]
        [           -z3*zet*xi*(-y3)*y1,            -z3*zet*(-x4)*x3*eta,  z4*(-x4)*x3*(-y3)*y1]
        [    -2*(-z3)*z1*xi*(-y3)*y1,     -2*(-z3)*z1*(-x4)*x3*eta,     -2*zet*(-x4)*x3*(-y3)*y1]
        ];
    return;
end
