% Draw a graphic representation.
%
% function draw (self, gv, context)
%
%
% Input arguments
% self = self
% gv = graphic viewer
% context = struct
% with mandatory fields
%    x=reference geometry field
%    u=displacement field
% with optional fields
%    facecolor = color of the faces, solid
%    colorfield =field with vertex colors
%       only one of the facecolor and colorfield may be supplied
%    shrink = shrink factor
%
function draw (self, gv, context)
    % these are all defined in clockwise order, which is what Matlab expects
    context.faces=[ 4     1     2     3
        2     1     5     6
        3     2     6     7
        4     3     7     8
        1     4     8     5
        6     5     8     7];
    context.edges= [1,2;2,3;3,4;4,1;5,6;6,7;7,8;8,5;1,5;2,6;3,7;4,8];
    node_pc=[-1,-1,-1;+1,-1,-1;+1,+1,-1;-1,+1,-1;...
        -1,-1,+1;+1,-1,+1;+1,+1,+1;-1,+1,+1;];
    if isfield(context,'shrink')
        c=ones(size(node_pc,1),1)*(sum(node_pc)/size(node_pc,1));
        pc=c+context.shrink*(node_pc-c);
        for j=1:size(pc,1)
            context.shrunk_pc_N{j}=bfun(self,pc(j,:));
        end
    end
    draw (self.gcellset_3_manifold, gv, context);
end

% this is the old implementation of the same method
%     % these are all defined in clockwise order, which is what Matlab expects
%     f =[ 4     1     2     3
%         2     1     5     6
%         3     2     6     7
%         4     3     7     8
%         1     4     8     5
%         6     5     8     7];
%     ef=[1,2,6,5;2,3,7,6;3,4,8,7;4,1,5,8];
%     % normally, we define faces with counterclockwise traversal of the vertices
%     % f=[1     4     3     2;...
%     %     1     2     6     5;...
%     %     2     3     7     6;...
%     %     3     4     8     7;...
%     %     4     1     5     8;...
%     %     5     6     7     8];
%     edgecolor = 'Black';
%     if (isfield(context,'edgecolor'))
%         edgecolor =context.edgecolor;
%     end
%     if (isfield(context,'colorfield'))
%     else
%         facecolor = 'red';
%         if (isfield(context,'facecolor'))
%             facecolor = context.facecolor;
%         end
%         edgecolor = 'Black';
%         if (isfield(context,'edgecolor'))
%             edgecolor = context.edgecolor;
%         end
%         options=struct ('facecolor',facecolor,'edgecolor',edgecolor);
%     end
%     if (isfield(context,'facealpha'))
%         options.facealpha = context.facealpha;
%     end
%     conns = get(self, 'conn'); % connectivity
%     shrink  = 1.0;
%     if isfield(context,'shrink')
%         shrink =context.shrink;
%     end
%     if (shrink==1)
%         nn =max(max(conns));
%         x = gather(context.x, 1:nn, 'values', 'noreshape'); % coordinates of nodes
%         u = gather(context.u, 1:nn, 'values', 'noreshape'); % coordinates of nodes
%         xu=x+u;
%         if (isfield(context,'colorfield'))
%             colors =gather(context.colorfield, 1:nn, 'values', 'noreshape'); % coordinates of nodes
%             options=struct ('colors',colors);
%         end
%         if (isfield(context,'edgecolor'))
%             options.edgecolor =context.edgecolor;
%         end
%         edgecolor='black';
%         options.edgecolor =edgecolor;
%         if ~strcmp(edgecolor,'none')
%             for i=1:size(ef,1)
%                 draw_polyline (gv, xu, conns(:,ef(i,:)), options);
%             end
%         end
%         options.edgecolor ='none';
%         for i=1:size(f,1)
%             draw_polygon (gv, xu, conns(:,f(i,:)), options);
%         end
%     else
%         for ac=1:size(conns,1)
%             conn =conns(ac,:);
%             x = gather(context.x, conn, 'values', 'noreshape'); % coordinates of nodes
%             u = gather(context.u, conn, 'values', 'noreshape'); % coordinates of nodes
%             if (isfield(context,'colorfield'))
%                 colors =gather(context.colorfield, conn, 'values', 'noreshape'); % coordinates of nodes
%                 options=struct ('colors',colors);
%             end
%             xu=x+u;
%             if isfield(context,'shrink')
%                 c=sum(xu,1)/8;
%                 xu=(1-context.shrink)*ones(8,1)*c+context.shrink*xu;
%             end
%             % c=f(:,1);f(:,1)=f(:,2);f(:,2)=c;
%             % c=f(:,3);f(:,3)=f(:,4);f(:,4)=c;
%             edgecolor='black';
%             options.edgecolor =edgecolor;
%             if ~strcmp(edgecolor,'none')
%                 for i=1:size(f,1)
%                     draw_polyline (gv, xu, f(i,:), options);
%                 end
%             end
%             options.edgecolor ='none';
%             for i=1:size(f,1)
%                 draw_polygon (gv, xu, f(i,:), options);
%             end
%         end
%     end
% end