% Draw a graphic representation.
%
% Input arguments
% self = self
% gv = graphic viewer
% context = struct
% with mandatory fields
%    geom=geometry field
% with optional fields
% facecolor = color of the faces, solid
% colorfield =field with vertex colors
% only one of the facecolor and colorfield may be supplied
%  shrink = shrink factor
%
function draw (self, gv, context)
    options=context;
    conns = get(self, 'conn'); % connectivity
    for i=1:size(conns,1)
        conn=conns(i,:);
        x = gather(context.x, conn, 'values', 'noreshape'); % coordinates of nodes
        u = gather(context.u, conn, 'values', 'noreshape'); % coordinates of nodes
        R = gather(context.R, conn, 'values', 'noreshape'); % rotations at nodes
        if (isfield(context,'colorfield'))
            colors =gather(context.colorfield, conn, 'values', 'noreshape'); % coordinates of nodes
            options.colors=[ones(4,1)*colors(1,:);ones(4,1)*colors(2,:)];
            options.facecolor = 'none';
        end
        % the following are approximations of the dimensions of a rectangular
        % cross-section that is roughly representative of a general cross-section
        % with the same moment of inertia and the same cross-sectional area
        h=sqrt(12*self.I2(i)/self.A(i));
        b=sqrt(12*self.I3(i)/self.A(i));
        T= local_coordinate_system (self,x,self.x1x2_vector(:,i));
        T1=reshape(R(1,:),3,3)*T;
        T2=reshape(R(2,:),3,3)*T;
        xc=zeros(8,3);
        xu=x+u;
        xc(1,:)=xu(1,:)-b/2*T1(:,2)'-h/2*T1(:,3)';
        xc(2,:)=xu(1,:)+b/2*T1(:,2)'-h/2*T1(:,3)';
        xc(3,:)=xu(1,:)+b/2*T1(:,2)'+h/2*T1(:,3)';
        xc(4,:)=xu(1,:)-b/2*T1(:,2)'+h/2*T1(:,3)';
        xc(5,:)=xu(2,:)-b/2*T2(:,2)'-h/2*T2(:,3)';
        xc(6,:)=xu(2,:)+b/2*T2(:,2)'-h/2*T2(:,3)';
        xc(7,:)=xu(2,:)+b/2*T2(:,2)'+h/2*T2(:,3)';
        xc(8,:)=xu(2,:)-b/2*T2(:,2)'+h/2*T2(:,3)';
        f=[1     4     3     2;...
            1     2     6     5;...
            2     3     7     6;...
            3     4     8     7;...
            4     1     5     8;...
            5     6     7     8];
        draw_polygon(gv, xc, f, options);
    end
end
