% Draw the color bar
%
% function draw_colorbar(self, options)
%
% options= Structure with optional fields:
% colormap=Colormap
% position=where to put it in the normalized coordinates
% minmax=minimum and maximum (also can be supplied as a field range)
% label= descriptive text
% 
function draw_colorbar(self, options)
    cmap=colormap;
    if isfield(options,'colormap')
        cmap = options.colormap;
    end
    color=[0, 0, 0];
    if isfield(options,'color')
        color = options.color;
    end
    FontName='Times';
    if isfield(options,'fontname')
        FontName = options.fontname;
    end
    Fontsize=12;
    if isfield(options,'fontsize')
        Fontsize = options.fontsize;
    end
    position =[0.869 0.15 0.05 0.7];
    if isfield(options,'position')
        position = options.position;
    end
    minmax = [0, 1];
    if isfield(options,'minmax')
        minmax= options.minmax;
    end
    if isfield(options,'range')
        minmax= options.range;
    end
    label = ' Data';
    if isfield(options,'label')
        label= options.label;
    end
    colormap(cmap);
    cbh=colorbar;
    set(cbh,...
        'Position',position,'YLim',minmax);
    YTick= get(cbh,'YTick');
    YTick=(YTick-minmax(1))./(minmax(2)-minmax(1));
    YTickLabel= get(cbh,'YTickLabel');
    m=size(YTickLabel,1);
    yl(1:m)=deal({' '});
    for i=1:m
        yl(i)={num2str(YTick(i)*(minmax(2)-minmax(1))+minmax(1))};
    end
    YTickLabel=yl;
    mtd=1/5/length(YTick);
    if abs(YTick(1)) >mtd
        YTick=[0,YTick];
        YTickLabel=cat(2,{num2str(minmax(1))},YTickLabel);
    end
    if abs(YTick(end)-1) >mtd
        YTick=[YTick,1];
        YTickLabel= cat(2,YTickLabel,{num2str(minmax(2))});
    end
    XTickLabel= get(cbh,'XTickLabel');
    m=size(XTickLabel,1);
    xl(1:m)=deal({' '});
    set(cbh,...
        'Position',position,'XColor',color,'YColor',color,...
        'CLim',[0, 1],'YLim',[0, 1],'YTick',YTick,'YTickLabel',YTickLabel,...
        'XTickMode','manual','XTickLabelMode','manual','XTick',[],'XTickLabel',xl);
    if (~isempty(Fontsize))
        set(cbh,'Fontsize',Fontsize);
    end
    if (~isempty(FontName))
        set(cbh,'fontname',FontName);
    end
   set(get(cbh,'XLabel'),'String',label);
    if (~isempty(Fontsize))
        set(get(cbh,'XLabel'),'Fontsize',Fontsize);
    end
    
end
