% Draw the coordinate planes of the global Cartesian coordinate system.
%
% function draw_planes(self, options)
%
% Draw the basis vector of the global Cartesian coordinate system as
% arrows, red, green, and blue.
% 
% options= optional structure, with fields
% length = length of the side of the squares that represent the planes
% facealpha= transparency
% origin = origin
function draw_planes(self, options)
    if ((nargin >1) && isstruct(options) && isfield(options,'length'))
        a=options.length;
    else
        ax=get(self.figure,'CurrentAxes');
        a=abs(get(ax,'XLim'));
        a=a+abs(get(ax,'YLim'));
        a=a+abs(get(ax,'ZLim'));
        a=sum(a)/20;
        options = struct('color', [0, 0, 0]);
    end
    facealpha=0.3;
    if (isfield(options,'facealpha'))
        facealpha= options.facealpha;
    end
    Origin=[0 0 0];
    if (isfield(options,'origin'))
        Origin= options.origin;
    end
    options.facecolor='red';
    options.edgecolor='none';
    options.facealpha=facealpha;
    draw_polygon(self, ones(4,1)*Origin+[[0 0 0]; [0 a 0]; [0 a a]; [0 0 a]], [1,2,3,4], options);
    draw_text(self, Origin+[a  0 0], 'X', options);
    options.facecolor='green';
    options.edgecolor='none';
    options.facealpha=facealpha;
    draw_polygon(self, ones(4,1)*Origin+[[0 0 0]; [a  0 0]; [a  0 a]; [0 0 a]], [1,2,3,4], options);
    draw_text(self, Origin+[0 a  0], 'Y', options);
    options.facecolor='blue';
    options.edgecolor='none';
    options.facealpha=facealpha;
    draw_polygon(self, ones(4,1)*Origin+[[0 0 0]; [a  0 0]; [a  a 0]; [0 a 0]], [1,2,3,4], options);
    draw_text(self, Origin+[0 0  a], 'Z', options);
    return;
end

