rho=1.21*1e-9;% mass density
c =343*1000;% millimeters per second
bulk= c^2*rho;
L=500;% length of the box, millimeters
A=2500; % cross-sectional area of the box
online_graphics= true;% plot the solution as it is computed?
scale=10;
n=8;% needs to be multiple of five
neigvs=5;
OmegaShift=10;

[fens,gcells] = L2_block(L,n,A); % Mesh
[fens,gcells] = L2_to_L3(fens,gcells,A); % Mesh

prop=property_viscous_fluid(struct('bulk',bulk,'rho',rho));
mater=mater_defor_ss_linviscel_triax (struct('property',prop));
feb = feblock_acoustics (struct (...
    'mater',mater,'gcells',gcells,...
    'integration_rule',simpson_1_3_rule));
point_gcells= gcellset_P1(struct('id',[1;,n+1],'conn',[1;n+1]));

geom = field(struct ('name',['geom'], 'dim', 1, 'fens',fens));
P = field(struct ('name',['P'], 'dim', 1,'nfens',get(geom,'nfens')));
P = numbereqns (P);

S = start (sparse_sysmat, get(P, 'neqns'));
S = assemble (S, acoustic_stiffness(feb, geom, P));
C = start (sparse_sysmat, get(P, 'neqns'));
C = assemble (C, acoustic_mass(feb, geom, P));


[W,Omega]=eigs(get(C,'mat')+OmegaShift*get(S,'mat'),get(S,'mat'),neigvs,'SM');
[Omegas,ix]=sort(diag(Omega)-OmegaShift);
fs=sqrt(Omegas)/(2*pi);

w=0*clone(P,'W');
for i=1:neigvs
    analyt_eigenvalue =sqrt((bulk/rho*((i-1)*pi/L) ^ 2))/2/pi;
    eigenvalue_errors(i) =...
        (fs(i)-analyt_eigenvalue)/analyt_eigenvalue;
    disp(['  Eigenvector ' num2str(i) ...
        ' eigenvalue ' num2str(fs(i)) ' versus '...
        num2str(analyt_eigenvalue)]);
    
    figure;
    w = scatter_sysvec(w, W(:,ix(i)));
    plot (gather (geom, (1:n+ 1),'values'), ...
        gather (w, (1:n+ 1),'values'),'ro-','linewidth', 3);
    hold on
    figure (gcf)
    pause(2); % next eigenvector
    
end