% Hemisphere in water, planar wave directed at an angle to the X. axis
% Formulation with fixed point iteration
function savefile=hemis_td_img_iter
    frequency=100; angle=0;
    %     pressuremag*R/(rhow*cw^2)./(2*pi*frequency/cw*R)*3/(2*rho/rhow+1);
    rho=1e-9;% mass density
    c = 1.4491e+006;% millimeters per second
    P_amplitude=10^9*1e-6*1e-6;% MPa
    rhos=2.5e-9;% mass density
    %     rhos=rho;% mass density
    maxiter=3;
    dt=1/frequency/15;
    tshift=0;
    tfinal=4*(1/frequency);
    graphics= ~true;% plot the solution as it is computed?
    saveimg= ~true;
    savevariables=~true;
    
    bulk= c^2*rho;
    nsteps=round(tfinal/dt)+1;
    omega=2*pi*frequency;
    wavenumber=omega/c;
    front_normal=[cos(pi/180*angle),sin(pi/180*angle),0]';
    wavevector=wavenumber*front_normal;
    uincA=P_amplitude/rho/c/omega;
    postfilename=[mfilename '-a' num2str(angle) '-f' num2str(frequency)];
    savefile=[postfilename  '.mat'];
    
    % % %     % Mesh
    [img,dx,dy,dz] = voximg_load('pw_hemis-nR3-22x22x22');
    img (img==3)=2;
    tol=min([dx,dy,dz])/1000;
    
    [t,v,tmid] = voximg_to_H8(img,[1,2]);
    v(:,1)=dx*v(:,1);v(:,2)=dy*v(:,2);v(:,3)=dz*v(:,3);
    
    fens =fenodeset(struct('id',(1:size(v,1))','xyz',v));
    gcells =gcellset_H8(struct('conn',double(t),'label',tmid));
    
    geom = field(struct ('name',['geom'], 'dim', 3, 'fens',fens));
    options =struct ('label', 1);
    Interiorl=gcell_select(fens,gcells,options) ;
    feb = feblock (struct ('mater',[], 'gcells',subset(gcells,Interiorl),...
        'integration_rule', gauss_rule (3,2)));
    V=measure(feb,geom,@(x)(1))
    S=measure(feb,geom,@(x)(x))
    CG=S/V
    I=measure(feb,geom,@(x)(norm(x-CG)^2*eye(3)-(x-CG)'*(x-CG)))
    M=V*rhos;
    K=I*rhos;
    
    smoothingoptions.method= 'laplace';
    smoothingoptions.npass=3;
    [ignore,v] =smoothing(t(find(tmid==1),:),v,smoothingoptions);
    fens =fenodeset(struct('id',(1:size(v,1))','xyz',v));
    
    disp( ['Mesh size: final ' num2str(size(t,1))])
    
    options =struct ('label', 2);
    Extl2=gcell_select(fens,gcells,options) ;
    options =struct ('label', 2);
    Extl3=gcell_select(fens,gcells,options) ;
    gcells=subset(gcells, [Extl2,Extl3]);
    connected = find_unconn_fens(fens, gcells);
    [fens, new_numbering] =compact_fens(fens, connected);
    gcells = renumber_gcell_conn(gcells, new_numbering);
    
    validate_mesh(fens, gcells)
    
    bdry_gcells = mesh_bdry(gcells, struct('other_dimension', 1));
    xyz=get(fens, 'xyz');
    % min(xyz)
    on=fenode_select(fens,struct ('box',[min(xyz),min(xyz)],'inflate',10*eps));
    options =struct ('flood',true,'startfen',on);
    Extl=gcell_select(fens,bdry_gcells,options) ;
    Intl=setdiff(1:count(bdry_gcells),Extl);
    target_gcells=subset(bdry_gcells,Intl);
    % gv=drawmesh({fens, subset(bdry_gcells,Extl)},'gcells','facecolor','w','facealpha',0.2);
    % gv=drawmesh({fens, subset(bdry_gcells,Intl)},'gv',gv,'gcells','facecolor','r','facealpha',0.7)
    
    % % %
    
    prop=property_viscous_fluid(struct('bulk',bulk,'rho',rho));
    mater=mater_defor_ss_linviscel_triax (struct('property',prop));
    feb = feblock_acoustics (struct (...
        'mater',mater,'gcells',gcells,...
        'integration_rule',gauss_rule(3,2)));
    function n= surface_normal(x,tangent)
        n= cross(tangent(:,1),tangent(:,2));
        n=n/norm(n);
    end
    sfeb = feblock_acoustics (struct (...
        'mater',mater,'gcells',target_gcells,...
        'integration_rule',gauss_rule(2,2),'N',@surface_normal));
    abcfeb = feblock_acoustics (struct (...
        'mater',mater,'gcells',subset(bdry_gcells,Extl),...
        'integration_rule',gauss_rule(2,2)));
    
    geom = field(struct ('name',['geom'], 'dim', 3, 'fens',fens));
    P = field(struct ('name',['P'], 'dim', 1,'nfens',get(geom,'nfens')));
    P = numbereqns (P);
    
    S = start (sparse_sysmat, get(P, 'neqns'));
    S = assemble (S, acoustic_stiffness(feb, geom, P));
    mS =get(S,'mat'); clear S
    C = start (sparse_sysmat, get(P, 'neqns'));
    C = assemble (C, acoustic_mass(feb, geom, P));
    mC =get(C,'mat'); clear C
    D = start (sparse_sysmat, get(P, 'neqns'));
    D = assemble (D, surface_acoustic_ABC(abcfeb, geom, P));
    mD =get(D,'mat'); clear D
    
    ForceF = field(struct ('name',['ForceF'], 'dim', 1,'nfens',3));
    ForceF = numbereqns (ForceF);
    
    TorqueF = field(struct ('name',['TorqueF'], 'dim', 1,'nfens',3));
    TorqueF = numbereqns (TorqueF);
    
    GF = start (sparse_sysmat, get(ForceF, 'neqns'), get(P, 'neqns'));
    GF = assemble(GF, pressure_to_resultant_force (sfeb, geom, P, ForceF));
    mGF =get(GF,'mat'); clear GF
    GT = start (sparse_sysmat, get(TorqueF, 'neqns'), get(P, 'neqns'));
    GT = assemble(GT, pressure_to_resultant_torque(sfeb, geom, P, TorqueF, CG));
    mGT =get(GT,'mat'); clear GT
%     mH = mGF'*(rho/M)*mGF+mGT'*(rho*inv(K))*mGT;
    
    % Solve
    P0 =0*P; % initially all pressure is zero
    clear P
    vP0= gather_sysvec(P0);
    vP1=0*vP0;
    vQ0= 0*vP0;
    vQ1=0*vQ0;
    t=0;
    P1 =P0;
    tTa1=[0;0;0];
    tAa1=[0;0;0];
    tTa0=tTa1; 
    tAa0=tAa1; 
    
    fit= force_intensity(struct('magn',@target_pressure_gradient));
    fiabc= force_intensity(struct('magn',@abc_pressure_gradient));
    
    pinc =0*P0; % initially all pressure is zero
    pincdd =0*P0; % initially all pressure acceleration is zero
    xyz =get(geom,'values');
    allnl=(1:count(fens));
    function recalculate_incident(t)
        arg=(-xyz*wavevector+omega*(t+tshift));% wave along the x-axis
        vpinc=P_amplitude*sin(arg);
        pinc= scatter(pinc,allnl,vpinc);
        vpincdd=-omega^2*vpinc;
        pincdd= scatter(pincdd,allnl,vpincdd);
        clear vpinc vpincdd
    end
    recalculate_incident(0);
    L0=-mS*gather_sysvec(pincdd)-mC*gather_sysvec(pinc);
    [Fresultant,Tresultant] = pressure_resultant (sfeb, geom, pinc+P1, CG);
    tTa1=Fresultant/M; tAa1=K\Tresultant;
    F = start (sysvec, get(P1, 'neqns'));
    F = assemble (F, pressure_flux_loads (sfeb, geom, P1, fit));
    F = assemble (F, pressure_flux_loads (abcfeb, geom, P1, fiabc));
    La0 =get(F, 'vec');
    
    tic; disp([' Factorize mA'  ])
    mA = (2/dt)*mS +mD +(dt/2)*mC;
    disp(['    nnz mA='  num2str(nnz(mA))])
    %     [D,dD,prm] = ldl(D,'vector');
    [mA,status,prm] = chol(mA,'lower','vector');
    if ( status ~= 0 ) error,end
    mAt= mA';
    disp(['    Factorized nnz mA='  num2str(nnz(mA))])
    disp(['      Done ' num2str(toc) ])
    
    if ( graphics )
        bfeb = sfeb;
        gv=graphic_viewer;
    end
    
    tTv=zeros(3,round(tfinal/dt)+1);
    tTu=tTv;tAv=tTv;tAu=tTv;    incTu=tTv;
    t=0;
    incTu(:,1)=-uincA*sin((-CG*wavevector+omega*(t+tshift)))*front_normal;
    step =0; snap=0;
    while t <=tfinal
        step = step  +1;
        tic; disp([' Time ' num2str(t) ' (' num2str(step) '/' num2str(round(tfinal/dt)+1) ')'])
        if graphics
            Camera =[-1.4380e+003 -3.0078e+003 -2.0797e+003  1.4759e+002  3.1662e+002  2.2190e+002            0            0  1.0000e+000  3.3580e-001];
            Camera =1.0e+003 *[ -1.4328   -3.0097   -2.0806    0.1528    0.3147    0.2210         0         0    0.0010    0.0001];
            wv = get(pinc+P0,'values');%
            dcm=data_colormap(struct ('range',P_amplitude*[-1,1]/10, 'colormap',bwr));
            colors=map_data(dcm, wv);
            colorfield = field(struct ('name',['cf'], 'dim', 3, 'data',colors));
            gv=reset (gv,[]);
            camset(gv,Camera);
            draw(bfeb,gv, struct ('x', geom,'u',0*geom,'colorfield',colorfield));
            draw_axes (gv,struct('length',2));
            pause(0.0055);
            if saveimg
                saveas(gcf, [mfilename '-' 'p'  '-i' n2s0p(snap) '.png'], 'png');
                %             saveas(gcf, [op{1} '-' 'Tiso'  '-t' n2s0p(pt) '.png'], 'png');
                %                 system ([' convert -resize 28% ' [postfile '-' 'Tiso'  '-i' n2s0p(img) 'nat.png'] ' ' [postfile '-' 'Tiso'  '-i' n2s0p(img) '-smallnat.png']])
                snap= snap+1;
            end
        end
        t=t+dt;
        recalculate_incident(t);
        L1=-mS*gather_sysvec(pincdd)-mC*gather_sysvec(pinc);
        crhs=(2/dt)*(mS*vQ0)-mD*vQ0-mC*(2*vP0+(dt/2)*vQ0)+L0+L1+La0;
        F = start (sysvec, get(P1, 'neqns'));
        F = assemble (F, pressure_flux_loads (abcfeb, geom, P1, fiabc));
        La1a =get(F, 'vec');
        for iter=1:maxiter
            La1 =-(mGF'*tTa1+mGT'*tAa1)*rho+La1a;
            rhs=crhs+La1;
            vQ1(prm) = mAt\(mA\rhs(prm));
            vP1 = vP0 + (dt/2)*(vQ0+vQ1);
            P1 = scatter_sysvec(P1, vP1);
            temporary=gather_sysvec(pinc+P1);
            Fresultant=mGF*temporary;
            Tresultant=mGT*temporary;
            tTa1=Fresultant/M; tAa1=K\Tresultant;
        end
            [Fresultant,Tresultant]
        vP0 = vP1;
        vQ0 = vQ1;
        P0 = P1;
        L0=L1;
        La0=La1;
        tTv(:,step+1)=tTv(:,step)+dt/2*(tTa0+tTa1);        
        tTu(:,step+1)=tTu(:,step)+dt/2*(tTv(:,step)+tTv(:,step+1));
        tAv(:,step+1)=tAv(:,step)+dt/2*(tAa0+tAa1);       
        tAu(:,step+1)=tAu(:,step)+dt/2*(tAv(:,step)+tAv(:,step+1));
        incTu(:,step+1)=-uincA*cos((-CG*wavevector+omega*(t+tshift)))*front_normal;
        tTa0=tTa1;
        tAa0=tAa1; 
    end% time integration loop
    tTu=tTu (:,1:step);
    tAu=tAu (:,1:step);
    incTu=incTu (:,1:step);
    clear mS mC mD mA mAt
    if ( savevariables )
        save (savefile);
    end
    
    t=linspace(0,size(tTu,2)*dt,size(tTu,2));
    figure;
    hold on
    plot(t,tTu(1,:),'rx-')
    plot(t,tTu(2,:),'gx--')
    plot(t,tTu(3,:),'bx:')
    xlabel(' Time'); ylabel(' Displacement of center of gravity'); set_graphics_defaults
    figure
    hold on
    plot(t,tAu(1,:),'rx-')
    plot(t,tAu(2,:),'go--')
    plot(t,tAu(3,:),'bs:')
    xlabel(' Time'); ylabel(' Rotation about center of gravity'); set_graphics_defaults
    
    function val=target_pressure_gradient(x,J)
        n=cross(J(:,1),J(:,2)); n=n/norm(n);
        val=-rho*dot(cross(tAa1,x-CG)+tTa1',n);
    end
    function val=abc_pressure_gradient(x,J)
        n=cross(J(:,1),J(:,2)); n=n/norm(n);
        arg=(-x*wavevector+omega*(t+tshift));
        val=P_amplitude*cos(arg)*(-dot(n,wavevector));
    end
    
    
end