% The sphere inside is oscillated from side to side, dipole-like.
% One can see the pressure waves coming off the sphere, leaving through the
% absorbing boundary condition surface.
function sphere_dipole_td1
    rho=1.21e-3*1e-9;% mass density of air
    c =343*1000;% sound speed of air, millimeters per second
    bulk= c^2*rho;
    a_amplitude=1;% millimeters per second
    R=50;% length of the box, millimeters
    Ro=400; % width of the box, millimeters
    tol=R/10000;
    P_amplitude=R/5*rho*a_amplitude;% MPa
    nref=2;%
    nlayers=18;
    frequency=1200; % Hz
    omega=2*pi*frequency;
    dt=1/frequency/20;
    tfinal=90*dt;
    nsteps=round(tfinal/dt)+1;
    graphics= true;% plot the solution as it is computed?
    
    [fens,gcells]=H8_sphere(R,nref);
    [fens1,gcells1] = mirror_mesh(fens, gcells, [-1,0,0], [0,0,0], @(c)c([1, 4, 3, 2, 5, 8, 7, 6]));
    [fens,gcells1,gcells2] = merge_meshes(fens1, gcells1, fens, gcells, tol);
    gcells=cat(gcells1,gcells2);
    
    bg=mesh_bdry(gcells);
    l=gcell_select(fens,bg,struct ('facing', true, 'direction', [0,1,1]));
    clear gcells
    % gv=drawmesh({fens,subset(bg,l)},'gcells','facecolor','white')
    % gv=drawmesh({fens,gcells1},'gcells','facecolor','white')
    [fens,gcells] = Q4_extrude(fens,subset(bg,l),nlayers,@(xyz, layer)(R+layer/nlayers*(Ro-R))*xyz/norm(xyz));
    connected = find_unconn_fens(fens, gcells);
    [fens, new_numbering] =compact_fens(fens, connected);
    gcells = renumber_gcell_conn(gcells, new_numbering);
    
    bg=mesh_bdry(gcells);
    clear options
    options. distance=R;
    options. inflate=tol;
    l1=gcell_select(fens, bg, options);
    clear options
    options. facing=true;
    options. direction=-[0,1,1];
    l2=gcell_select(fens, bg, options);
    sphere_gcells=subset(bg,intersect(l1,l2));
    
    %     gv=drawmesh({fens,sphere_gcells},'gcells','facecolor','red')
    
    outerl=gcell_select(fens,bg,struct ('facing', true, 'direction', [0,1,1]));
    outer_gcells=subset(bg,outerl);
    
    prop=property_viscous_fluid(struct('bulk',bulk,'rho',rho));
    mater=mater_defor_ss_linviscel_triax (struct('property',prop));
    feb = feblock_acoustics (struct (...
        'mater',mater,'gcells',gcells,...
        'integration_rule',gauss_rule(3,2)));
    sfeb = feblock_acoustics (struct (...
        'mater',mater,'gcells',sphere_gcells,...
        'integration_rule',gauss_rule(2,2)));
    abcfeb = feblock_acoustics (struct (...
        'mater',mater,'gcells',outer_gcells,...
        'integration_rule',gauss_rule(2,2)));
    
    geom = field(struct ('name',['geom'], 'dim', 3, 'fens',fens));
    P = field(struct ('name',['P'], 'dim', 1,'nfens',get(geom,'nfens')));
    P = numbereqns (P);
    
    S = start (sparse_sysmat, get(P, 'neqns'));
    S = assemble (S, acoustic_stiffness(feb, geom, P));
    mS =get(S,'mat');
    C = start (sparse_sysmat, get(P, 'neqns'));
    C = assemble (C, acoustic_mass(feb, geom, P));
    mC =get(C,'mat');
    D = start (sparse_sysmat, get(P, 'neqns'));
    D = assemble (D, surface_acoustic_ABC(abcfeb, geom, P));
    mD =get(D,'mat');
    
    
    % Solve
    P0 =0*P; % initially all pressure is zero
    clear P
    vP0= gather_sysvec(P0);
    vP1=0*vP0;
    vQ0= 0*vP0;
    vQ1=0*vQ0;
    t=0;
    P1 =P0;
    
    fi= force_intensity(struct('magn',@dipole_pressure_gradient));
    F = start (sysvec, get(P1, 'neqns'));
    F = assemble (F, pressure_flux_loads (sfeb, geom, P1, fi));
    La0 =get(F, 'vec');
    
    mA = (2/dt)*mS +mD +(dt/2)*mC;
    
    
    if ( graphics )
        bfeb = feblock(struct ('gcells',bg));
        gv=graphic_viewer;
    end
    
    
    step =0;
    while t <=tfinal
        step = step  +1;
        tic; disp([' Time ' num2str(t) ' (' num2str(step) '/' num2str(round(tfinal/dt)+1) ')'])
        if graphics
            Camera =1.0e+003 * [-1.4303   -3.0332   -2.0483    0.1553    0.2912    0.2533         0         0    0.0010    0.0066];
            wv = get(P1,'values');
            dcm=data_colormap(struct ('range',P_amplitude*[-1,1], 'colormap',bwr));
            colors=map_data(dcm, wv);
            colorfield = field(struct ('name',['cf'], 'dim', 3, 'data',colors));
            gv=reset (gv,[]);
            camset(gv,Camera);
            draw(bfeb,gv, struct ('x', geom,'u',0*geom,'colorfield',colorfield));
            draw_axes (gv,struct('length',R));
            pause(0.55);
        end
        t=t+dt;
        F = start (sysvec, get(P1, 'neqns'));
        F = assemble (F, pressure_flux_loads (sfeb, geom, P1, fi));
        La1 =get(F, 'vec');
        vQ1 = mA\((2/dt)*(mS*vQ0)-mD*vQ0-mC*(2*vP0+(dt/2)*vQ0)+La0+La1);
        vP1 = vP0 + (dt/2)*(vQ0+vQ1);
        vP0 = vP1;
        vQ0 = vQ1;
        P1 = scatter_sysvec(P1, vP1);
        P0 = P1;
        La0 =La1;
    end
    function val=dipole_pressure_gradient(x)
        n=-x/norm(x);% Outer normal to the interior sphere
        val=-rho*a_amplitude*sin(omega*t)*n(1);
    end
end