%% Da Deppo, Schmidt circular arc 

% Parameters:
E=1e7;
nu=0.3;
b=3; h=7;% cross-sectional dimensions

nel=32;
% Mesh
ang=107.5*2;
rad=100;
x=[];
xyz=[];
id=[];
for i=0:nel
    a=(i/nel*ang-17.5)/360*2*pi;
    x =[rad*cos(a) rad*sin(a) 0];
    id(end+1)=i+1; xyz=[xyz;x];
end
fens=fenodeset(struct ('id',id,'xyz',xyz));

id=[];
conns = [];
for i=1:nel
    id(end+1)=i; conns= [conns;[i i+1]];
end
gcells = gcellset_beam3(struct ('id',id,'conn',conns, ...
    'A',b*h,'I2',b^3*h/12,'I3',b*h^3/12,'IT',b^2*h^2/2,...
    'x1x2_vector',[0 0 1]));


prop = property_linel_iso (struct('E',E,'nu',nu));
mater = mater_defor_ss_linel_triax (struct('property',prop));
% Finite element block
feb = feblock_defor_ss_beam3(struct ('mater',mater, 'gcells',gcells));
% Geometry
geom = field(struct ('name',['geom'], 'dim', 3, 'fens',fens));
% Define the displacement field
ur  = field(struct ('name',['ur'], 'dim', 6, 'data', zeros(get(geom,'nfens'),6)));

% Apply EBC's
ebc_fenids=[1];
ebc_prescribed=[1];
ebc_comp=[];
ebc_val=ebc_fenids*0;
ur  = set_ebc(ur, ebc_fenids, ebc_prescribed, ebc_comp, ebc_val);
ur  = apply_ebc (ur);
for i=1:3
ebc_fenids=nel+1;
ebc_prescribed=[1];
ebc_comp=[i];
ebc_val=ebc_fenids*0;
ur  = set_ebc(ur, ebc_fenids, ebc_prescribed, ebc_comp, ebc_val);
ur  = apply_ebc (ur);
end

% Number equations
ur  = numbereqns (ur);
neqns=get(ur, 'neqns');

% Assemble the system matrix
ems = stiffness(feb, geom, ur);
K = dense_sysmat;
K = start (K, neqns);
K = assemble (K, ems);
% Load
F = sysvec;
F = start (F, neqns);
% evs = loads(nodal_load(struct ('id',round(nel/2)+1,'dir',[1],'magn',[-50000])), ur);
% F = assemble (F, evs);
evs = loads(nodal_load(struct ('id',round(nel/2)+1,'dir',[2],'magn',[-50000])), ur);
F = assemble (F, evs);
% Solve
a = get(K,'mat');
b = get(F,'vec');
x = a\b;
ur = scatter_sysvec(ur, x);
% get(ur,'values')

% Plot
gv=graphic_viewer;
gv=reset (gv,[]);
scale=1;
u=slice(ur,(1:3),'u');
rot=slice(ur,(4:6),'rot');
draw(feb, gv, struct ('x', geom, 'u', 0*u, 'rot', 0*rot, 'facecolor','none'));
draw(feb, gv, struct ('x', geom, 'u', scale*u, 'rot', scale*rot, 'facecolor','red'));

xlabel('X');
ylabel('Y');
zlabel('Z');
hold on

for i=1:length(fens)
    draw(fens(i),gv, struct ('x', geom, 'u', 0*u, 'color','blue','offset',0.35));
end
draw_axes(gv, struct([]));

pause (2);
for scale=sin(-pi/2+(0:1:51)/21*2*pi)
    clf;
    title ('Animation of deflection');
    gv=reset (gv,struct ('limits',[-120 120 -30 120 -20 20]));
    draw(feb,gv, struct ('x', geom, 'u', 0*u, 'rot', 0*rot, 'facecolor','none'));
    draw(feb,gv, struct ('x', geom, 'u', (scale+1)*u, 'rot', (scale+1)*rot, 'facecolor','red'));
    pause(0.01);
end
