%%  Simply-supported beam with distributed load. Stiff  axis bending.
%% 
% Parameters:
E=30002*1000;%30002ksi
b=2;%in
h=18;%in
L=240;% in
q=[0;0;1;];% 1200 lbf/ft
%% 
% Cross-sectional properties
A=b*h;%in^2
I2=b*h^3/12;%cm^4 
I3=b^3*h/12;%cm^4 
nu=0.;

%% 
% Exact deformation under the load

deflex=5*norm(q)*L^4/(384*E*I2);

%% 
% The problem is essentially solved exactly with a single element.
n=2;
Parameters=struct ('A',A,'I2',I2,'I3',I3,'IT',I2+I3,...
    'x1x2_vector',[1 0 0], 'label',1);
[fens,gcells] = Beam_member([[0 -L/2 0];[0 L/2 0]],2*n,@gcellset_beam3,Parameters);

%% 
% Material properties
prop = property_linel_iso (struct('E',E,'nu',nu));
mater = mater_defor_ss_linel_triax (struct('property',prop));
%% 
% Finite element block
feb = feblock_defor_ss_beam3(struct ('mater',mater, 'gcells',gcells));


%% 
% Construct the requisite fields, geometry and displacement

geom = field(struct ('name',['geom'], 'dim', 3, 'fens',fens));
% Define the displacement field
ur  = field(struct ('name',['ur'], 'dim', 6, 'data', zeros(get(geom,'nfens'),6)));

% Apply EBC's
ebc_fenids=fenode_select(fens, struct('box', update_box([],[0 -L/2 0])));
ebc_prescribed=[1];
ebc_comp=[1,2,3,5];% cylindrical 
ebc_val=ebc_fenids*0;
ur  = set_ebc(ur, ebc_fenids, ebc_prescribed, ebc_comp, ebc_val);
ebc_fenids=fenode_select(fens, struct('box', update_box([],[0 L/2 0])));
ebc_prescribed=[1];
ebc_comp=[1,2,3,5];% cylindrical
ebc_val=ebc_fenids*0;
ur  = set_ebc(ur, ebc_fenids, ebc_prescribed, ebc_comp, ebc_val);
ur  = apply_ebc (ur);

% Number equations
ur  = numbereqns (ur);
neqns=get(ur, 'neqns');

%% 
% Assemble the global discrete system

% Assemble the system matrix
K = start (sparse_sysmat, neqns);
K = assemble (K, stiffness(feb, geom, ur));
% Load
F = start (sysvec, neqns);
fi = force_intensity(struct('magn' ,q ));
evs = distrib_loads(feb, geom, ur, fi);
F = assemble (F, evs);

%% 
% Solve the system and report the results
ur = scatter_sysvec(ur, K\F);
utip=gather(ur,fenode_select(fens, struct('box', update_box([],[0 0 0]))),'values');
disp(['Normalized deflection at the midpoint =' num2str(utip(3)/deflex,'%27.15e\n')])
disp(['   Results normalized by analytical values'])
disp(['      midpoint deflection =' num2str(deflex,'%27.15e\n')])

%% 
% Graphic display of the mesh and the deformed beam
gv=graphic_viewer;
gv=reset (gv,[]);
scale=25000;
u=slice(ur,(1:3),'u');
rot=slice(ur,(4:6),'rot');
draw(feb, gv, struct ('x', geom, 'u', 0*u, 'rot', 0*rot, 'facecolor','none'));
draw(feb, gv, struct ('x', geom, 'u', scale*u, 'rot', scale*rot, 'facecolor','m'));
draw_local_coordinates(feb, gv, struct ('x', geom, 'u', 0*u, 'rot', 0*rot, 'length',25));

xlabel('X');
ylabel('Y');
zlabel('Z');
hold on

draw(fens,gv, struct ('x', geom, 'u', 0*u, 'color','blue','offset',0.35));
draw_axes(gv, struct([]));
