% Circular arc, static deflection out of plane
% Parameters:
E=1e7;
nu=0.3;
b=7; h=3;

nel=10;
% Mesh
ang=107.5*2;
rad=100;
xyz=[];
id=[];
for i=0:nel
    id(end+1)=i+1;
    xyz=[xyz;[rad*cos(i/nel*ang/360*2*pi) rad*sin(i/nel*ang/360*2*pi) 0]];
end
fens=fenodeset(struct ('id',id,'xyz',xyz));

id=[];
conns = [];
for i=1:nel
    id(end+1)=i; conns= [conns;[i i+1]];
end
gcells = gcellset_beam3(struct ('id',id,'conn',conns, ...
        'A',b*h,'I2',b*h^3/12,'I3',b^3*h/12,'IT',b^2*h^2/2,...
        'x1x2_vector',[0,0,1]));
    
ebc_fenids=[1];
ebc_prescribed=[1];
ebc_comp=[];
ebc_val=ebc_fenids*0;

% Material
prop = property_linel_iso (struct('E',E,'nu',nu));
mater = mater_defor_ss_linel_triax (struct('property',prop));
% Finite element block
feb = feblock_defor_ss_beam3(struct ('mater',mater, 'gcells',gcells));
% Geometry
geom = field(struct ('name',['geom'], 'dim', 3, 'fens',fens));
% Define the displacement field
ur  = field(struct ('name',['ur'], 'dim', 6, 'data', zeros(get(geom,'nfens'),6)));

% Apply EBC's
ur  = set_ebc(ur, ebc_fenids, ebc_prescribed, ebc_comp, ebc_val);
ur  = apply_ebc (ur);
% Number equations
ur  = numbereqns (ur);
neqns=get(ur, 'neqns');

% Assemble the system matrix
K = start (dense_sysmat, neqns);
K = assemble (K, stiffness(feb, geom, ur));
% Load
F = start (sysvec, neqns);
evs = loads(nodal_load(struct ('id',nel+1,'dir',[3],'magn',[10000])), ur);
F = assemble (F, evs);
% Solve
ur = scatter_sysvec(ur, K\F);
% get(ur,'values')

% Plot
gv=graphic_viewer;
gv=reset (gv,[]);
scale=2;
u=slice(ur,(1:3),'u');
rot=slice(ur,(4:6),'rot');
draw(feb, gv, struct ('x', geom, 'u', 0*u, 'rot', 0*rot, 'facecolor','none'));
draw_local_coordinates(feb, gv, struct ('x', geom, 'u', 0*u, 'rot', 0*rot, 'length',25));
draw(feb, gv, struct ('x', geom, 'u', scale*u, 'rot', scale*rot, 'facecolor','red'));

xlabel('X');
ylabel('Y');
zlabel('Z');
hold on

draw(fens,gv, struct ('x', geom, 'u', 0*u, 'color','blue','offset',0.35));
 
draw_axes(gv, struct([]));