
%% Spinning shaft about an offset axis.
%
function [rotary_speeds,umnum,umana]=spinning_shaft_free_mc
    % Parameters:
    E=30002*1000;%Young's modulus 30002ksi
    nu=0.3;% Poisson ratio
    radius_internal=2.5/2;%in
    radius_external=6.25/2;%in
    L=45*12;% in
    g=32.17*12; % in/second^2
    rho=490/12^3/g; %lb/ft^3
    eccentricity=0.1;% axis eccentricity in inches
    zeta= 0.05; % damping ratio for mode 1
    zeta_1= zeta; % damping ratio for mode 1
    zeta_2= zeta; % damping ratio for mode 2
    % Stiffness proportional damping is selected.
    omega_1=2*pi*1.832; % computed previously for this structure in stival_4_7
    omega_2=2*pi*7.328; % computed previously for this structure in stival_4_7
    % zeta = (Rayleigh_damping_stiffness/omega_1)/2
    Rayleigh_damping_mass= 1/(omega_2^2-omega_1^2)*[omega_2^2,-omega_1^2]*2*[zeta_1*omega_1;zeta_2*omega_2];
    Rayleigh_damping_stiffness= 1/(omega_2^2-omega_1^2)*[-1,1]*2*[zeta_1*omega_1;zeta_2*omega_2];
    
    % Newmarkg = 1/2; Newmarkb = 1/4*(1/2+Newmarkg)^2;
    Newmarkg = 0.8; Newmarkb = 1/4*(1/2+Newmarkg)^2;
    maxit=70;
    
    
    %%
    % Output control
    Report =@(x) (disp(x) );
    graphics= false;
    
    X=[[-L/2 eccentricity 0];[L/2 eccentricity 0]];
    
    %%
    % Cross-sectional properties
    A=pi*(radius_external^2-radius_internal^2);%in^2
    I2=pi/4*(radius_external^4-radius_internal^4);;%cm^4
    I3=pi/4*(radius_external^4-radius_internal^4);;%cm^4
    IT=pi/2*(radius_external^4-radius_internal^4);
    
    %%
    %
    n=2;
    Parameters=struct ('A',A,'I2',I2,'I3',I3,'IT',IT,...
        'x1x2_vector',[0 1 0], 'label',1);
    [fens,gcells] = Beam_member(X,2*n,@gcellset_beam3,Parameters);
    midn=fenode_select(fens, struct('box', update_box([],mean(X))));
    end1n=fenode_select(fens, struct('box', update_box([],X(1,:))));
    end2n=fenode_select(fens, struct('box', update_box([],X(2,:))));
    
    %%
    % Material properties
    prop = property_linel_iso (struct('E',E,'nu',nu,'rho',rho));
    mater = mater_defor_ss_linel_triax (struct('property',prop));
    %%
    % Finite element block
    feb = feblock_defor_ss_beam3_corot(struct ('mater',mater, 'gcells',gcells,...
        'Rayleigh_damping_stiffness',Rayleigh_damping_stiffness,...
        'Rayleigh_damping_mass',Rayleigh_damping_mass));
    
    rotary_speeds=25:25:200;
    umnum=[];
    umana=[];
    for rotary_speed=rotary_speeds;%rotary speed, RPM
        freq=rotary_speed/60;% circular frequency in hertz
        Omega=freq*2*pi;% Angular frequency of spin
        
        %%
        % Time integration parameters
        Tper=1/freq;
        dt=Tper/41.4;
        tend=(6+(rotary_speed/200)*50)*Tper;
        
        e=4*eccentricity/pi;
        beta=Omega/omega_1;
        uampl_analyt=e*beta^2/sqrt((1-beta^2)^2+(2*zeta*beta^2));
        
        spin_vector= [Omega;0;0];
        spin_center= [0 0 0];
        
        %%
        % Construct the requisite fields, geometry and displacement
        geom = field(struct ('name',['geom'], 'dim', 3, 'fens',fens));
        % Define the displacement field
        ur  = field(struct ('name',['ur'], 'dim', 6, 'data', zeros(get(geom,'nfens'),6)));
        Rfield = field(struct ('name',['Rfield'], 'dim', 9, 'data', ones(get(geom,'nfens'),1)*reshape(eye(3),1,9)));
        
        % Apply EBC's
        ur  = set_ebc(ur, end1n, 1, [1,2,3,4], 0);
        ur  = set_ebc(ur, end2n, 1, [1,2,3,4], 0);
        ur  = apply_ebc (ur);
        
        % Number equations
        ur  = numbereqns (ur);
        neqns=get(ur, 'neqns');
        
        
        if graphics, gv=graphic_viewer; end
        
        if graphics,
            gv=reset(clear(gv,[]),[]);
            %                     camset(gv,cam);
            view(2);
            displ=slice(ur,(1:3),'displ');
            rot=slice(ur,(4:6),'rot');
            draw(feb, gv, struct ('x', geom, 'u', displ, 'rot', rot, 'facecolor','none'));
            pause(0.5);
            %                     cam =camget(gv);
        end
        
        % initial conditions
        ur = ur*0; % zero out the displacement
        vr = ur*0; % zero out the Velocity
        xyz = gather(geom,(1:count(fens)),'values','noreshape');
        vrxyz = gather(vr,(1:count(fens)),'values','noreshape');
        vrxyz =[-xyz*skewmat(spin_vector), ones(count(fens),1)*spin_vector'];
        vr= scatter (vr,(1:count(fens)),vrxyz);
        ar = ur*0; % zero out the Acceleration
        utol = 1e-9*get(ur, 'neqns');
        
        K = start (sparse_sysmat, get(ur, 'neqns'));
        M = start (sparse_sysmat, get(ur, 'neqns'));
        C = start (sparse_sysmat, get(ur, 'neqns'));
        
        Mid_uy=[];
        
        t=0; % time=load magnitude
        while (t <= tend)
            t = t + dt;
            Report(['Time ' num2str(t) ]); % pause
            % Initialization
            ur1 = ur; % guess
            dur = 0*ur; % this will hold displacement increment
            end1nu1=(rotmat(spin_vector*t)-eye(3))*(X(1,:)-spin_center)' ;
            ur1  = set_ebc(ur1, end1n, 1, 1:3, [end1nu1']);
            ur1  = set_ebc(ur1, end2n, 1, 1:3, [end1nu1' ]);
            ur1 = apply_ebc(ur1);
            Rfield1=Rfield;
            dro1 = 0*ur1;
            dro1  = set_ebc(dro1, end1n, 1, 4:6, [spin_vector'*dt]);
            dro1  = set_ebc(dro1, end2n, 1, 4:6, [spin_vector'*dt]);
            dro1 = apply_ebc(dro1);
            Rfield1=update_rotation_field(Rfield1,slice(dro1,(4:6),'rot'));
            dur = apply_ebc(dur);
            ar1 = -(1/Newmarkb/dt)*vr -(1/2-Newmarkb)/Newmarkb*ar;
            vr1 = vr + dt*((1-Newmarkg)*ar + Newmarkg*ar1);
            urpv =gather_sysvec(ur +dt*vr +(dt^2/2*(1-2*Newmarkb))*ar);
            vrpv =gather_sysvec(vr +(dt*(1-Newmarkg))*ar);
            
            iter=1;
            while 1
                feb1=feb;
                F = start (sysvec, get(ur, 'neqns'));
                [feb,evs] = restoring_force(feb,geom,ur1,Rfield1);       % Internal forces
                F = assemble (F, evs);
                K =assemble (K,cat(2,stiffness(feb, geom, ur1,Rfield1),stiffness_geo(feb, geom, ur1,Rfield1)));
                Km=get(K, 'mat');
                M =assemble (M,mass(feb, geom, ur1,Rfield1));
                Mm=get(M, 'mat');
                C =assemble (C,damping_proportional(feb, geom, ur1,Rfield1));
                Cm=get(C, 'mat');
                rhs=get(F,'vec')...
                    +Mm*((-1/(Newmarkb*dt^2))*gather_sysvec(ur1)+(1/(Newmarkb*dt^2))*urpv)...
                    +Cm*((-Newmarkg/Newmarkb/dt)*gather_sysvec(ur1)+(Newmarkg/Newmarkb/dt)*urpv-vrpv);
                dur = scatter_sysvec(dur, (Km+(Newmarkg/Newmarkb/dt)*Cm+(1/(Newmarkb*dt^2))*Mm)\rhs); % Displacement increment
                %                 max(abs(get(F,'vec')))
                ur1 = ur1 + dur;   % increment displacement
                vr1 = vr1 + (Newmarkg/Newmarkb/dt)*dur;
                ar1 = ar1 + (1/Newmarkb/dt^2)*dur;
                Rfield1=update_rotation_field(Rfield1,slice(dur,(4:6),'rot'));
                Report(['   It. ' num2str(iter) ': ||du||=' num2str(norm(dur))]);
                if (max(abs(gather(dur,[],'values'))) < utol) break; end;% convergence check
                if (iter >maxit)% bailout for failed convergence
                    warning( [' Possible failed convergence']);
                end
                K = reinitialize (K);
                M = reinitialize (M);
                C = reinitialize (C);
                iter=iter+1;
            end
            [feb,ignore] = restoring_force(feb,geom,ur1,Rfield1);        % final update
            Report(['    Converged for t=' num2str(t)]); % pause
            ur = ur1;       % update the displacement
            vr = vr1;       % update the velocities
            ar = ar1;       % update the accelerations
            Rfield=Rfield1; % update the rotations
            
            umid=gather(ur,midn,'values');
            Mid_uy=[Mid_uy;umid(2:3)'-end1nu1(2:3)'];
            
            if graphics
                displ=slice(ur,(1:3),'displ');
                rot=slice(ur,(4:6),'rot');
                gv=reset(gv,[]);
                draw(feb, gv, struct ('x', geom, 'u', displ, 'R', Rfield1, 'facecolor','g'));
                draw(feb, gv, struct ('x', geom, 'u', 0*displ, 'rot', 0*rot, 'facecolor','none'));
                draw_local_coordinates(feb, gv, struct ('x', geom, 'u', displ, 'R', Rfield1, 'length',12.5));
                draw_axes (gv,struct('length',L/30 ));
                camset(gv, 1.0e+003 *[-4.7025   -0.1968    0.1667    0.0018    0.0086    0.0022         0         0    0.0010    0.0004]);
                drawnow;
            else
                if (mod(size(Mid_uy,1),2)==0)
                    plot(t/Tper,Mid_uy(end,1),'m+'); hold on
                    plot(t/Tper,Mid_uy(end,2),'bx'); hold on
                    title(['rotary speed ' num2str(rotary_speed)])
                    drawnow;
                end
            end
        end
        
        %done for the rotary speed
        umnum=[umnum,max([abs(Mid_uy(end-3*round(Tper/dt):end,1));abs(Mid_uy(end-3*round(Tper/dt):end,2))])]
        umana=[umana,uampl_analyt];
    end