%% Spinning shaft about an offset axis. 
% 
function spinning_shaft_x
% Parameters:
E=30002*1000;%30002ksi
nu=0.3;
radius_internal=2.5/2;%in
radius_external=6.25/2;%in
L=45*12;% in
g=32.17*12; % in/second^2
rho=490/12^3/g; %lb/ft^3
Omega=0.2*2*pi;
spin_vector= [Omega;0;0];
spin_center= [0 0 0];
eccentricity=0.1;
X=[[-L/2 eccentricity 0];[L/2 eccentricity 0]];
graphics= false;
dt=1/Omega/60;
tend=300*dt;
% Newmarkg = 1/2; Newmarkb = 1/4*(1/2+Newmarkg)^2;
Newmarkg = 0.6; Newmarkb = 1/4*(1/2+Newmarkg)^2;
maxit=12;
neigvs=2;
Report =@(x) (disp(x) );
%%
% Cross-sectional properties
A=pi*(radius_external^2-radius_internal^2);%in^2
I2=pi/4*(radius_external^4-radius_internal^4);;%cm^4 
I3=pi/4*(radius_external^4-radius_internal^4);;%cm^4 
IT=pi/2*(radius_external^4-radius_internal^4);


%% 
% 
n=2;
Parameters=struct ('A',A,'I2',I2,'I3',I3,'IT',IT,...
    'x1x2_vector',[0 1 0], 'label',1);
[fens,gcells] = Beam_member(X,2*n,@gcellset_beam3,Parameters);
midn=fenode_select(fens, struct('box', update_box([],mean(X))));
end1n=fenode_select(fens, struct('box', update_box([],X(1,:))));
end2n=fenode_select(fens, struct('box', update_box([],X(2,:))));

%% 
% Material properties
prop = property_linel_iso (struct('E',E,'nu',nu,'rho',rho));
mater = mater_defor_ss_linel_triax (struct('property',prop));
%% 
% Finite element block
feb = feblock_defor_ss_beam3_corot(struct ('mater',mater, 'gcells',gcells));


%% 
% Construct the requisite fields, geometry and displacement

geom = field(struct ('name',['geom'], 'dim', 3, 'fens',fens));
% Define the displacement field
ur  = field(struct ('name',['ur'], 'dim', 6, 'data', zeros(get(geom,'nfens'),6)));
Rfield = field(struct ('name',['Rfield'], 'dim', 9, 'data', ones(get(geom,'nfens'),1)*reshape(eye(3),1,9)));

% Apply EBC's
ur  = set_ebc(ur, end1n, 1, [1,2,3,4], 0);
ur  = set_ebc(ur, end2n, 1, [2,3,4], 0);
ur  = apply_ebc (ur);

% Number equations
ur  = numbereqns (ur);
neqns=get(ur, 'neqns');


if graphics, gv=graphic_viewer; end

if graphics,
    gv=reset(clear(gv,[]),[]);
    %                     camset(gv,cam);
    view(2);
    displ=slice(ur,(1:3),'displ');
    rot=slice(ur,(4:6),'rot');
    draw(feb, gv, struct ('x', geom, 'u', displ, 'rot', rot, 'facecolor','none'));
    pause(0.5);
    %                     cam =camget(gv);
end

ur = ur*0; % zero out the displacement
vr = ur*0; % zero out the Velocity
xyz = gather(geom,(1:count(fens)),'values','noreshape');
vrxyz = gather(vr,(1:count(fens)),'values','noreshape');
vrxyz =[-xyz*skewmat(spin_vector), zeros(count(fens),1)*spin_vector'];
vr= scatter (vr,(1:count(fens)),vrxyz);
ar = ur*0; % zero out the Acceleration
utol = 1e-10*get(ur, 'neqns');


    K = start (sparse_sysmat, neqns);
    K = assemble (K, stiffness(feb, geom, ur, Rfield));
    
    
    M = start (sparse_sysmat, neqns);
    M = assemble (M, mass(feb, geom, ur, Rfield));
    
    options.tol =1e-26;
    options.maxit = 5000;
    options.disp = 0;
    [W,Omega]=eigs(get(K,'mat'),get(M,'mat'), neigvs,'sm', options);
    [Omegas2,ix]=sort((diag(Omega))) ;
    Omegas2=somel(diag(Omega),ix);
    Frequencies= sqrt(Omegas2)/2/pi;
    
    %%
    % Computed frequencies
    disp([' Modal analysis: frequencies for ' num2str(n) ' elements'])
    for i=1:neigvs
        disp(['  Eigenvector ' num2str(i) ' frequency ' num2str(Frequencies(i)) '  [Hz]']);
    end
    disp(' ')
    