%% Spinning of a shaft about an offset axis. 
% 
% Parameters:
E=30002*1000;%30002ksi
nu=0.3;
radius_internal=2.5/2;%in
radius_external=6.25/2;%in
L=45*12;% in
g=32.17*12; % in/second^2
rho=490/12^3/g; %lb/ft^3
Omega=160*2*pi/60;
spin_vector= [Omega;0;0];
spin_center= [0 0 0];
eccentricity=4;
graphics= false;
dt=1/Omega/20;
tend=400*dt;
% Newmarkg = 1/2; Newmarkb = 1/4*(1/2+Newmarkg)^2;
Newmarkg = 0.6; Newmarkb = 1/4*(1/2+Newmarkg)^2;
maxit=12;
Report =@(x) (disp(x) );
%%
% Cross-sectional properties
A=pi*(radius_external^2-radius_internal^2);%in^2
I2=pi/4*(radius_external^4-radius_internal^4);;%cm^4 
I3=pi/4*(radius_external^4-radius_internal^4);;%cm^4 
IT=pi/2*(radius_external^4-radius_internal^4);

%% 
% 
n=6;
Parameters=struct ('A',A,'I2',I2,'I3',I3,'IT',IT,...
    'x1x2_vector',[0 1 0], 'label',1);
X=[[-L/2 eccentricity 0];[L/2 eccentricity 0]];
[fens,gcells] = Beam_member(X,2*n,@gcellset_beam3,Parameters);
midn=fenode_select(fens, struct('box', update_box([],mean(X))));
end1n=fenode_select(fens, struct('box', update_box([],X(1,:))));
end2n=fenode_select(fens, struct('box', update_box([],X(2,:))));

%% 
% Material properties
prop = property_linel_iso (struct('E',E,'nu',nu,'rho',rho));
mater = mater_defor_ss_linel_triax (struct('property',prop));
%% 
% Finite element block
feb = feblock_defor_ss_beam3_corot(struct ('mater',mater, 'gcells',gcells));


%% 
% Construct the requisite fields, geometry and displacement

geom = field(struct ('name',['geom'], 'dim', 3, 'fens',fens));
% Define the displacement field
ur  = field(struct ('name',['ur'], 'dim', 6, 'data', zeros(get(geom,'nfens'),6)));
Rfield = field(struct ('name',['Rfield'], 'dim', 9, 'data', ones(get(geom,'nfens'),1)*reshape(eye(3),1,9)));

% Apply EBC's
ur  = set_ebc(ur, end1n, 1, [], 0);
ur  = set_ebc(ur, end2n, 1, [], 0);
ur  = apply_ebc (ur);

% Number equations
ur  = numbereqns (ur);
neqns=get(ur, 'neqns');


if graphics, gv=graphic_viewer; end

if graphics,
    gv=reset(clear(gv,[]),[]);
    %                     camset(gv,cam);
    view(2);
    displ=slice(ur,(1:3),'displ');
    rot=slice(ur,(4:6),'rot');
    draw(feb, gv, struct ('x', geom, 'u', displ, 'rot', rot, 'facecolor','none'));
    pause(0.5);
    %                     cam =camget(gv);
end

% Now comes the nonlinear solution
tup = 1;
ur = ur*0; % zero out the displacement
vr = ur*0; % zero out the Velocity
ar = ur*0; % zero out the Acceleration
utol = 1e-10*get(ur, 'neqns');

K = start (sparse_sysmat, get(ur, 'neqns'));
M = start (sparse_sysmat, get(ur, 'neqns'));
Mems=mass(feb, geom, ur, Rfield);% Constant mass matrix
mat=get(Mems,'mat');
for  j = 1:length(mat)
     mat{j} =mat{j}/(Newmarkb*dt^2);
end
Mems=set(Mems,'mat',mat);
clear M

Mid_uy=[];

t=0; % time=load magnitude
while (t <= tend)
    Report(['Time ' num2str(t) ]); % pause
    % Initialization
    ur1 = ur; % guess
    dur = 0*ur; % this will hold displacement increment
    ur1 = apply_ebc(ur1);
    Rfield1=Rfield;
    dur = apply_ebc(dur);
    ar1 = -(1/Newmarkb/dt)*vr -(1/2-Newmarkb)/Newmarkb*ar;
    vr1 = vr + dt*((1-Newmarkg)*ar + Newmarkg*ar1);
    F0 = start (sysvec, get(ur, 'neqns'));
    arp =-(1/Newmarkb/dt^2)*ur -(1/Newmarkb/dt)*vr -((1/2-Newmarkb)/Newmarkb)*ar;
    F0 = assemble (F0, inertial_force(feb, geom, (-Newmarkb*dt^2)*arp, Mems));
    
    iter=1;
    while 1
        feb1=feb;
        F = F0;
        [feb,evs] = restoring_force(feb,geom,ur1,Rfield1);       % Internal forces
        F = assemble (F, evs);
        F = assemble (F, inertial_force(feb, geom, (-1)*ur1, Mems));
        F = assemble (F, centrifugal_loads(feb, geom, ur1,Rfield1, spin_vector, spin_center));
        ems=cat(2,stiffness(feb, geom, ur1,Rfield1),stiffness_geo(feb, geom, ur1,Rfield1),Mems);
        K =assemble (K,ems);
        dur = scatter_sysvec(dur, K\F); % Displacement increment
        %                 max(abs(get(F,'vec')))
        ur1 = ur1 + dur;   % increment displacement
        vr1 = vr1 + (Newmarkg/Newmarkb/dt)*dur;
        ar1 = ar1 + (1/Newmarkb/dt^2)*dur;
        Rfield1=update_rotation_field(Rfield1,slice(dur,(4:6),'rot'));
        K = reinitialize (K);
        Report(['   It. ' num2str(iter) ': ||du||=' num2str(norm(dur))]);
        if (max(abs(gather(dur,[],'values'))) < utol) break; end;% convergence check
%         if (iter >maxit)% bailout for failed convergence
%             error( [' Possible failed convergence']);
%         end
        iter=iter+1;
    end
    [feb,ignore] = restoring_force(feb,geom,ur1,Rfield1);        % final update
    Report(['    Converged for t=' num2str(t)]); % pause
    ur = ur1;       % update the displacement
    vr = vr1;       % update the velocities
    ar = ar1;       % update the accelerations
    Rfield=Rfield1; % update the rotations
    
    umid=gather(ur,midn,'values');
    Mid_uy=[Mid_uy,umid(2)];
    
    if graphics 
        displ=slice(ur,(1:3),'displ');
        rot=slice(ur,(4:6),'rot');
        gv=reset(gv,[]);
        draw(feb, gv, struct ('x', geom, 'u', displ, 'R', Rfield1, 'facecolor','g'));
        draw(feb, gv, struct ('x', geom, 'u', 0*displ, 'rot', 0*rot, 'facecolor','red'));
%         draw_local_coordinates(feb, gv, struct ('x', geom, 'u', displ, 'R', Rfield1, 'length',12.5));
        draw_axes (gv,struct('length',L/30 ));
        camset(gv, 1.0e+003 *[-4.7025   -0.1968    0.1667    0.0018    0.0086    0.0022         0         0    0.0010    0.0004]);
        drawnow;
    else
        plot(t,Mid_uy(end),'+'); hold on
        drawnow;
    end
    
    t = t + dt;
end
