kappa=[1.8 0; 0 1.8]; % conductivity matrix
Q=4.5; % uniform heat source
R=2.5;
num_integ_pts=1; % 1-point quadrature
fens = fenodeset(struct('id',1:3','xyz',[[R*cos(0*pi/180),R*sin(0*pi/180)];...
    [R*cos(10*pi/180),R*sin(10*pi/180)];...
    [0,0]]));
gcells = gcellset_T3(struct('id',1,'conn',[1,2,3],'other_dimension', 1));
prop=property_diffusion(struct('conductivity',kappa,'source',Q));
mater=mater_diffusion (struct('property',prop));
feb = feblock_diffusion (struct ('mater',mater,...
    'gcells',gcells,...
    'integration_rule',tri_rule(num_integ_pts)));
geom = field(struct('name',['geom'], 'dim', 2, 'fens',fens));
theta=field(struct('name',['theta'], 'dim', 1, 'nfens',...
    get(geom,'nfens')));
fenids=[fenode_select(fens,struct('distance',R-0.02,'from',[0 0],...
    'inflate', 0.01))];
fenids= setdiff(1:count(fens),fenids);
prescribed=ones(length(fenids),1);
comp=[];
val=zeros(length(fenids),1)+273.15;
theta = set_ebc(theta, fenids, prescribed, comp, val);
theta = apply_ebc (theta);
theta = numbereqns (theta);
K = start (sparse_sysmat, get(theta, 'neqns'));
K = assemble (K, conductivity(feb, geom, theta));
F = start (sysvec, get(theta, 'neqns'));
F = assemble (F, source_loads(feb, geom, theta));
F = assemble(F, nz_ebc_loads_conductivity(feb, geom, theta));
theta = scatter_sysvec(theta, get(K,'mat')\get(F,'vec'));
% Plot
gv=graphic_viewer;
gv=reset (gv,[]);
T=get(theta,'values');
dcm=data_colormap(struct('range',[min(T),max(T)],'colormap',hot));
colorfield=field(struct ('name', ['colorfield'], 'data',...
    map_data(dcm, T)));
geomT=field(struct ('name', ['geomT'], ...
    'data',[get(geom,'values'), get(theta,'values')-273.15]));
draw(gcells, gv, struct ('x',geomT, 'u',0*geomT,...
    'colorfield',colorfield, 'shrink',1.0));
draw(gcells, gv, struct ('x',geom, 'u',0*geom, ...
    'facecolor','none'));
axis equal vis3d
% set(gca,'DataAspectRatio', [1, 1, 0.5*(max(T)/R)])
zlabel('Temperature [degree Celsius]')
(get(theta,'values')-273.15)
assignin('caller','faesor_test_passed',(abs((5.168770190135945-somel(get(theta,'values'),3))<1e-12)))
