disp('Example MPARTNEOH: large strain nonlinear analysis; neo-hookean material');
% Parameters:
E=100;
nu=0.4999;
ebcs=[];
for i=[49 50 53 54 59 60 61 62 65 66] % clamped end
    for j=1:3
        ebcs=[ebcs [i 1 j 0]'];
    end
end
Fmag=20;
loaddir=2;
nincr = 5;
scale=1;
stressscale=scale/20;

% Mesh
[fens,gcells] = mpart;
% Material
prop = property_linel_iso (struct('E',E,'nu',nu));
mater = mater_defor_neohook(struct('property',prop));
% Finite element block
feb = feblock_defor_nonlinear (struct ('mater',mater, 'gcells',gcells, 'integration_rule',gauss_rule(3, 2)));
% Geometry
geom = field(struct ('name',['geom'], 'dim', 3, 'fens',fens));
% Define the displacement field
u   = 0*geom; % zero out
% Apply EBC's
u   = set_ebc(u, ebcs(1,:), ebcs(2,:), ebcs(3,:), ebcs(4,:));
u   = apply_ebc (u);
% Number equations
u   = numbereqns (u);
% Assemble the system matrix
ems = stiffness(feb, geom, u,u);
K = start (dense_sysmat, get(u, 'neqns'));
K = assemble (K, ems);
% Load
F = start (sysvec, get(u, 'neqns'));
for i = [13 16]
    F = assemble (F, loads(nodal_load(struct ('id',i,'dir',[loaddir],'magn',[-Fmag])), u));
end
F = finish(F);
% Solve
a = get(K,'mat');
b = get(F,'vec');
x = a\b;
u = scatter_sysvec(u, x);
% Plot
gv=graphic_viewer;
gv=reset(clear(gv,[]),[]);
draw(feb,gv, struct ('x', geom,'u',u, 'facecolor','red'));
disp('Done with linear solution: hit return'); % pause


ulin = u; %  linear displacement solution

% Now comes the nonlinear solution
tup = 1;
u = u*0; % zero out the displacement
utol = 1e-12;
% utol = 1e+1;

t=0; % time=load magnitude
incr=1;
while (incr <= nincr)
    t = t + tup / nincr;
    % Initialization
    u1 = u; % guess
    u1 = apply_ebc(u1);
    du = 0*u; % this will hold displacement increment
    du = apply_ebc(du);
    iter=1;
    gv=reset(clear(gv,[]),[]);
    draw(feb,gv, struct ('x', geom,'u',u, 'facecolor','red'));
    while 1
        feb1=feb;                                           % Make a copy of the state
        F = start (sysvec, get(u, 'neqns'));
        for i = [13 16]
            F = assemble (F, loads(nodal_load(struct ('id',i,'dir',[loaddir],'magn',[-Fmag*t])), u));
        end
        [feb1,evs] = restoring_force(feb1,geom,u1,u);       % Internal forces
        F = assemble (F, evs);
        F = finish(F);
        K = start (dense_sysmat, get(u, 'neqns'));
        K = assemble (K, stiffness(feb1, geom, u1,u));    % Current material stiffness
        K = assemble (K, stiffness_geo(feb1, geom, u1,u));   % Current geometric stiffness
        % Displacement increment
        x = get(K,'mat') \ get(F,'vec');;
        du = scatter_sysvec(du, x);
        u1 = u1 + du;                                       % increment displacement
        draw(feb1,gv, struct ('x', geom,'u',u1, 'facecolor','none'));
        disp(['   Iteration ' num2str(iter) ': ||du||=' num2str(norm(du))]);
        if (norm(du) < utol) break; end;                    % convergence check
        %         break;
        pause(1)
        iter=iter+1;
    end
    [feb,ignore] = restoring_force(feb,geom,u1,u);        % final update
    disp(['Increment ' num2str(incr) ': Converged for t=' num2str(t)]); % pause
    u = u1;                                               % update the displacement
    gv=reset(clear(gv,[]),[]);
    draw(feb,gv, struct ('x', geom,'u',u, 'facecolor','red'));
    draw(feb,gv, struct ('x', geom,'u',ulin, 'facecolor','none'));
    incr = incr + 1;
end
