% Thick plate, small displacement, small strain analysis
disp('Timoshenko: clamped square plate with center load');
% !  Timoshenko: clamped square plate with center load
% ! w_max = 0.0056 * P * a^2 / (E * h^3 / (12 * (1 - nu^2))) =
% !     2.44608 for P=10, a=2, E=1e9, h=0.001, nu=0.3
% ! full model (not reduced by symmetry) -- 289 (displ) + 68 (lagr) unknowns
% ! reduced -- 81 (displ) + 54 (lagr)

% Parameters:
P=2; a=2; E=1e9; h=0.05; nu=0.3;
w_max = 0.0056 * P * a^2 / (E * h^3 / (12 * (1 - nu^2)))
graphics =~false; % graphic output
% Mesh
neqs=[]; normalized_deflections = [];
for shift= [-0.24   -0.2000   -0.1250   -0.0625    0.0000    0.0625    0.1250    0.2000    0.24]
for nt=2
    for na =4:4
        scale = 0.5/w_max;
        prop = property_linel_iso (struct('E',E,'nu',nu));
        mater = mater_defor_ss_linel_triax (struct('property',prop));
        [fens,gcells] = Q4_block (a/2,a/2,2,2, []);
        xy=get(fens, 'xyz' );
        xy(5,:)=xy(5,:)+shift*a;
        fens=set(fens, 'xyz',xy);
        nlayers=nt;
        [fens,gcells] = Q4_extrude(fens,gcells,nlayers,@(xyz, layer)[xyz, layer*h/nlayers]);
        
%       tag='H20';   [fens,gcells] = H8_to_H20(fens,gcells);   ir=gauss_rule(3, 2); febh =@feblock_defor_ss;
%       tag='H27';   [fens,gcells] = H8_to_H27(fens,gcells);   ir=gauss_rule(3, 3); febh =@feblock_defor_ss;
        tag='NICE-H27'; [fens,gcells] = H8_to_H27(fens,gcells);  ir=tensprod_nq_rule (struct('dim',3,'order',2)); febh =@feblock_defor_ss_nice;
%         tag='NICE-H64'; [fens,gcells] = H8_to_H64(fens,gcells);  ir=tensprod_nq_rule (struct('dim',3,'order',3)); febh =@feblock_defor_ss_nice;
        % Finite element block
        feb = febh (struct('mater',mater,'gcells',gcells,...
            'integration_rule',ir));
%                          mesh{1}=fens;
%                     mesh{2}=gcells;
%                     drawmesh({fens;gcells}); view(3); pause%(1)

        % Geometry
        geom = field(struct ('name',['geom'], 'dim', 3, 'fens',fens));
        % Define the displacement field
        u   = 0*geom; % zero out
        % Apply EBC's
        ebc_fenids=fenode_select (fens,struct('box',[0 0 0 a/2 0 h],'inflate',h/100));
        ebc_prescribed=ebc_fenids*0+1;
        ebc_comp=ebc_fenids*0+1;
        ebc_val=ebc_fenids*0;
        u   = set_ebc(u, ebc_fenids, ebc_prescribed, ebc_comp, ebc_val);
        ebc_fenids=fenode_select (fens,struct('box',[0 a/2 0 0 0 h],'inflate',h/100));
        ebc_prescribed=ebc_fenids*0+1;
        ebc_comp=ebc_fenids*0+2;
        ebc_val=ebc_fenids*0;
        u   = set_ebc(u, ebc_fenids, ebc_prescribed, ebc_comp, ebc_val);
        ebc_fenids=fenode_select (fens,struct('box',[0 a/2 a/2 a/2 0 h],'inflate',h/100));
        ebc_prescribed=ebc_fenids*0+1;
        ebc_comp=[];
        ebc_val=ebc_fenids*0;
        u   = set_ebc(u, ebc_fenids, ebc_prescribed, ebc_comp, ebc_val);
        ebc_fenids=fenode_select (fens,struct('box',[a/2 a/2 0 a/2 0 h],'inflate',h/100));
        ebc_prescribed=ebc_fenids*0+1;
        ebc_comp=[];
        ebc_val=ebc_fenids*0;
        u   = set_ebc(u, ebc_fenids, ebc_prescribed, ebc_comp, ebc_val);
        u   = apply_ebc (u);
        % Number equations
        u   = numbereqns (u);
        % Assemble the system matrix
        ems = stiffness(feb, geom, u);
        K = start (sparse_sysmat, get(u, 'neqns'));
        K = assemble (K, ems);
        % Load
        F = start (sysvec, get(u, 'neqns'));
        corn=fenode_select (fens,struct('box',[0 0 0 0 0 h],'inflate',h/100));
        evs = loads(nodal_load(struct ('id',corn,'dir',[3],'magn',[-P/4/length(corn)])), u);
        F = assemble (F, evs);
        % Solve
        u = scatter_sysvec(u, K\F);
        ucorn= gather(u, corn,'values','noreshape');
        nd=abs(mean(ucorn(:,3))/w_max)
        %toc
        normalized_deflections = [normalized_deflections,nd];
        neqs = [neqs get(u, 'neqns')];
        if graphics
            gv=graphic_viewer;
            gv=reset (gv,[]);
            for i=1:length (gcells)
                draw(gcells(i), gv, struct ('x',geom,'u', scale*u, 'facecolor','none'));
            end
            camset(gv, [2.9385   -2.9337    7.3859    0.5201    0.3346   -0.2607         0         0    1.0000    9.7999]);
            set_graphics_defaults;
            saveas(gcf,[mfilename '-shift' num2str(shift) '.eps'],'eps2c');
        end
    end
end
end
plot([-0.24   -0.2000   -0.1250   -0.0625    0.0000    0.0625    0.1250    0.2000    0.24], normalized_deflections/(normalized_deflections(5)),'kd-.','linewidth',3); hold on
if (~ graphics )
plot(neqs, normalized_deflections,'ks-.','linewidth',3); hold on
end