% Mesh of a rectangle located in a given range, Q4 elements
%
% function [fens,gcells] = Q4_block(Length,Width,nL,nW,options)
%
% <0,Length> x <0,Width>
% Divided into elements: nL, nW in the first, second (x,y).
% options = structure with fields recognized by the constructor of the
%       Q4 finite element
% 
% See the visual gallery by running test_block
% 
function [fens,gcells] = Q4_block(Length,Width,nL,nW,options)
    nL = ceil(nL); nW = ceil(nW);
    nnodes=(nL+1)*(nW+1);
    ncells=(nL)*(nW);
    xs=zeros(nnodes,2);
    conns=zeros(ncells,4);
    if ~isstruct(options)
        other_dimension = options; clear options;
        options.other_dimension = other_dimension;
    end

    f=1;
    for j=1:(nW+1)
        for i=1:(nL+1)
            xyz=[(i-1)*Length/nL (j-1)*Width/nW];
            xs(f,:)=xyz;
            f=f+1;
        end
    end
    fens=fenodeset(struct('id',(1:nnodes)','xyz',xs));

    gc=1;
    for i=1:nL
        for j=1:nW
            nn=node_numbers(i,j,nL,nW);
            conns(gc,:) =nn;
            gc=gc+1;
        end
    end
    options.conn =conns;
    gcells =gcellset_Q4(options);
end

function nn=node_numbers(i,j,nL,nW)
    f=(j-1)*(nL+1)+i;
    nn=[f (f+1)  f+(nL+1)+1 f+(nL+1)];
    return;
end
