% Convert a mesh of triangle T3 (three-node) to triangle T6.
%
% function [fens,gcells] = T3_to_T6(fens,gcells,options)
%
% options =attributes recognized by the constructor gcell_T6
function [fens,gcells] = T3_to_T6(fens,gcells,options)
    if ~isstruct(options)
        options = struct('id',1);
    end
    nedges=3;
    ec = [1, 2; 2, 3; 3, 1];
    % make a search structure for edges
    edges={};
    conns = get (gcells,'conn');
    for i= 1:count(gcells)
        conn=conns(i,:);
        for J = 1:nedges
            ev=conn(ec(J,:));
            anchor=min(ev);
            if length(edges)<anchor
                edges{anchor}=[];
            end
            edges{anchor}=unique([edges{anchor} max(ev)]);
        end
    end
    % now generate new node number for each edge
    n=count(fens);
    nodes=edges;
    for i= 1:length(edges)
        e=edges{i};
        for J = 1:length(e)
            n=n+1;
            e(J)=n;
        end
        nodes{i}=e;
    end
    nfens=n;% number of nodes in the original mesh plus number of the edge nodes
    xyz=get(fens,'xyz');
    xyz =zeros(n,size(xyz,2));
    xyz(1:count(fens),:) = get(fens,'xyz');
    id =zeros(n,1); 
    id(1:count(fens),:) = get(fens,'id');
    % calculate the locations of the new nodes
    % and construct the new nodes
    for i= 1:length(edges)
        e=edges{i};
        n=nodes{i};
        for J = 1:length(e)
            xyz(n(J),:)=(xyz(i,:)+xyz(e(J),:))/2;
            id(n(J),:)=n(J);
        end
    end
    nfens=count(fens);% number of nodes in the original mesh plus number of the edge nodes
    fens=  set(fens,'id',id);
    fens=  set(fens,'xyz',xyz);
    % construct new geometry cells
    nconns=zeros(count(gcells),6);
    nc=1;
    for i= 1:count(gcells)
        conn = conns(i,:);
        econn=zeros(1,nedges);
        for J = 1:nedges
            ev=conn(ec(J,:));
            anchor=min(ev);
            e=edges{anchor};
            n=nodes{anchor};
            econn(J)=n(find(e==max(ev)));
        end
        nconns(nc,:) =[conn econn];
        nc= nc+ 1;
    end
    gcells=gcellset_T6(struct('id',(1:count(gcells))','conn',nconns)) ;
end
