% General smoothing of meshes.
%
% function [t,v] =smoothing(t,v,options)
%
% Fields of the structure options
% method='laplace' or 'taubin'
% f=boundary faces
% bv=boundary vertices
% bv_from_f=compute boundary of vertices from the boundary faces, true or
% false
% npass=how many passes of smoothing?
function [t,v] =smoothing(t,v,options)
    iv=v;
    fvn = [];
    bv = [];
    bv_from_f = false;
    f= [];
    npass =2;
    method =@taubin_smoother;
    if (exist('options','var'))
        if (isfield(options,'bv'))
            bv= options.bv;
        end
        if (isfield(options,'f'))
            f= options.f;
        end
        if (isfield(options,'bv_from_f'))
            bv_from_f= options.bv_from_f;
        end
        if (isfield(options,'npass'))
            npass= options.npass;
        end
        if (isfield(options,'method'))
            if ( strcmp(options.  method,'laplace'))
                method =@laplace_smoother;
            elseif ( strcmp(options.  method,'taubin'))
                method =@taubin_smoother;
            end
        end

    end

    if (isempty(bv))
        bv=0*v(:,1);
    end
    if (bv_from_f)
        if (isempty(f))
            f = t4util_bdry(t); % Extract the boundary
            bv(f)=1;
        end
    end

    % find neighbors for the Volume connections
    vn =  t4util_vertex_neighbors([],t,v);

    % Smoothing considering only surface connections
    fvn = t4util_vertex_neighbors([],f,v);
    fv =  method(v,fvn,0*bv,npass,0.5,-0.5);

    % Smoothing considering all connections through the volume
    v =  method(v,vn,bv,npass,0.5,-0.5);

    % Correction of the vertices of the surface
    if (~isempty(fvn))
        for i= 1:length(fvn)
            if (~isempty(fvn {i}))
                v(i,:)= fv(i,:);
            end
        end
    end
end

function nv =  taubin_smoother(v,vn,bv,npass,lambda,mu)
    nv=v;
    for I= 1:npass
        o=randperm(length(vn));
        damping_factor=lambda;
        for k= 1:length(vn)
            r=o(k);
            n=setdiff(vn{r},r);
            if (~isempty(n)) && (~bv(r))
                nv(r,:)=(1-damping_factor)*v(r,:)+ damping_factor*sum(v(n,:))/length(n);
            end
        end
        v=nv;
        damping_factor=mu;
        for k= 1:length(vn)
            r=o(k);
            n=setdiff(vn{r},r);
            if (~isempty(n)) && (~bv(r))
                nv(r,:)=(1-damping_factor)*v(r,:)+ damping_factor*sum(v(n,:))/length(n);
            end
        end
        v=nv;
    end
end

function nv =  laplace_smoother(v,vn,bv,npass,lambda,mu)
    damping_factor=lambda;
    for I= 1:npass
        nv=v;
        o=randperm(length(vn));
        for k= 1:length(vn)
            r=o(k);
            n=setdiff(vn{r},r);
            if (~isempty(n)) && (~bv(r))
                nv(r,:)=damping_factor*v(r,:)+ (1-damping_factor)*sum(v(n,:))/length(n);
            end
        end
        v=nv;
    end

end